
# React Hooks



import NpmLogo from '@site/static/img/npm-logo.png';

<div class="row margin-bottom--md">
  <div class="col">
    <a href="https://www.npmjs.com/package/@dynatrace-sdk/react-hooks" target="_blank" rel="noopener noreferrer"> <span class="margin-right--xs">@dynatrace-sdk/react-hooks</span> <img className="no-zoom" width="20px" src={NpmLogo} /> </a>
  </div>
  <div class="col" style={{textAlign: 'right'}}>
    <a href="https://www.npmjs.com/package/@dynatrace-sdk/react-hooks/v/1.4.1" target="_blank" rel="noopener noreferrer">v1.4.1</a>
  </div>
</div>

```bash
npm install @dynatrace-sdk/react-hooks
```


### Why should I use this package?

This package simplifies interactions with Grail, documents, and app states in React by encapsulating complex state management. It offers easy-to-use hooks that integrate seamlessly with client packages, enhancing developer productivity and usability in React applications.

### Simplified data querying from Grail

Traditionally, querying data in Dynatrace involves using the client-query package and managing complex React state. The `useDql` hook in this package streamlines this process. The following example showcases how to fetch data with a DQL query:

```javascript
const { data, error, isLoading } = useDql('fetch logs');
```

This hook is fully compatible with the parameters used in the `queryExecute` function of the `@dynatrace-sdk/client-query` package.

For instance, to limit the number of results returned:

```javascript
const { data, error, isLoading, refetch } = useDql(
  {
    query: 'fetch logs',
    maxResultRecords: 2000,
  }
);
```

You can delay the execution of the query until a user clicks on a button by passing additional query options to the hook:

```javascript
const { data, error, isLoading, refetch } = useDql('fetch logs', { enabled: false });

function onClickQuery() {
  refetch();
}
```

You should add appropriate scopes to your app's configuration based on the query type. For more details, refer to the [Bucket and table permissions in Grail documentation](https://www.dynatrace.com/support/help/platform/grail/assign-permissions-in-grail).

### Interacting with documents and app states

Beyond DQL queries, our hooks facilitate interactions with documents and app state. They allow control over immediate or deferred query execution.

```javascript
const { data, error, isLoading } = useDocument({ id: documentId }, { autoFetch: true });
```

For creating, updating, or deleting documents or app state, an explicit execute call is necessary:

```javascript
const { data, execute, error } = useCreateDocument();

function onClickCreateDocument() {
  execute(DOCUMENT_DATA);
}
```

Depending on your interaction type, add these scopes to your app configuration:

| Function | Scope |
| ----- | ----- |
| Document read | document:documents:read |
| Document write/update | document:documents:write |
| Document delete | document:documents:delete |
| State read | state:app-states:read |
| State write | state:app-states:write |
| State delete | state:app-states:delete |
| User state read | state:user-app-states:read |
| User state write | state:user-app-states:write |
| User state delete | state:user-app-states:delete |

### Simplified Use of Davis® Analyzers

Leveraging Davis® analyzers traditionally involves complex state management and polling logic, alongside the `@dynatrace-sdk/client-davis-analyzers` package. The `useAnalyzer` hook in this package makes this process much more straightforward:

```javascript
const { data, error, isLoading } = useAnalyzer({
  analyzerName: 'dt.statistics.GenericForecastAnalyzer',
  body: {
    timeSeriesData: {
      expression: query,
    },
  },
});
```

This hook supports all the parameters available in the executeAnalyzer method from the `@dynatrace-sdk/client-davis-analyzers` package.

To defer the execution of the analyzer until a user action, like a button click, configure the hook with additional options:

```javascript
const { data, error, isLoading, refetch } = useAnalyzer({
  analyzerName: 'dt.statistics.GenericForecastAnalyzer',
  body: {
    timeSeriesData: {
      expression: query,
    },
  }, 
  {
    autoFetch: false,
    autoFetchOnUpdate: true,
  }
});

function onExecuteAnalyzer() {
  refetch();
}
```
In your app's configuration, include the necessary scope:

| Function | Scope |
| ----- | ----- |
| Use analyzer | davis:analyzers:execute |

### App functions

The useAppFunction hook is the simplest way to call app functions. As the other hooks in this package, it provides state handling for loading and error states:

```javascript
const { data, error, isLoading } = useAppFunction({ name: 'functionName', data: 'data' });
```

Sometimes you want to delay the execution of the app function until a user clicks on a button. This can be achieved by passing additional options to the hook:

```javascript
const { data, error, isLoading, refetch } = useAppFunction({ name: 'functionName', data: 'data' }, { autoFetch: false, autoFetchOnUpdate: false });

function onClick() {
  refetch();
}
```


## Hooks



### useAccessorPermissionsV2

<div class="padding-bottom--md">
<strong>useAccessorPermissionsV2(params,options?): Object</strong>

<div class="padding-left--md">

Load accessor permissions.



</div>


#### Parameters

| Name | Type | Description |
| --- | --- | --- |
|params<sup>*required</sup>|AccessorPermissionsParamsV2|Parameters for the client sdk call. |
|options|HookOptions|Additional options for the react hook. |








</div>





### useAllUsersPermissionsV2

<div class="padding-bottom--md">
<strong>useAllUsersPermissionsV2(params,options?): Object</strong>

<div class="padding-left--md">

Load all-users permissions.



</div>


#### Parameters

| Name | Type | Description |
| --- | --- | --- |
|params<sup>*required</sup>|LoadAllUsersPermissionsParamsV2|Parameters for the client sdk call. |
|options|HookOptions|Additional options for the react hook. |








</div>





### useAnalyzer

<div class="padding-bottom--md">
<strong>useAnalyzer(params,options?): Object</strong>

<div class="padding-left--md">

Execute an analyzer with the given data.



</div>


#### Parameters

| Name | Type | Description |
| --- | --- | --- |
|params<sup>*required</sup>|AnalyzerParams|Parameters for the client sdk call. |
|options|HookOptions|Additional options for the react hook. |








</div>





### useAppFunction

<div class="padding-bottom--md">
<strong>useAppFunction(params,options?): Object</strong>

<div class="padding-left--md">

Call specified app function by name.



</div>


#### Parameters

| Name | Type | Description |
| --- | --- | --- |
|params<sup>*required</sup>|AppFunctionParams|App function params. |
|options|HookOptions|Additional options for the react hook. |








</div>





### useAppState

<div class="padding-bottom--md">
<strong>useAppState(params,options?): Object</strong>

<div class="padding-left--md">

Gets app state



</div>


#### Parameters

| Name | Type | Description |
| --- | --- | --- |
|params<sup>*required</sup>|AppStateParams|Parameters for the client sdk call. |
|options|HookOptions|Additional options for the react hook. |








</div>





### useAppStates

<div class="padding-bottom--md">
<strong>useAppStates(params,options?): Object</strong>

<div class="padding-left--md">

List app states



</div>


#### Parameters

| Name | Type | Description |
| --- | --- | --- |
|params<sup>*required</sup>|AppStatesParams|Parameters for the client sdk call. |
|options|HookOptions|Additional options for the react hook. |








</div>





### useCreateDocument

<div class="padding-bottom--md">
<strong>useCreateDocument(): Object</strong>

<div class="padding-left--md">

Create a new document.



</div>







</div>





### useCreatePermissionsV2

<div class="padding-bottom--md">
<strong>useCreatePermissionsV2(): Object</strong>

<div class="padding-left--md">

Create a new permissions.



</div>







</div>





### ~~useCreateSettings~~

> ⚠️ **Deprecated**
> Use V2 instead.

<div class="padding-bottom--md">
<strong>useCreateSettings(): Object</strong>

<div class="padding-left--md">

Create a new setting.



</div>







</div>





### useCreateSettingsV2

<div class="padding-bottom--md">
<strong>useCreateSettingsV2(): Object</strong>

<div class="padding-left--md">

Create a new setting.



</div>







</div>





### useDeleteAccessorPermissionsV2

<div class="padding-bottom--md">
<strong>useDeleteAccessorPermissionsV2(): Object</strong>

<div class="padding-left--md">

Delete the accessor permissions



</div>







</div>





### useDeleteAllUsersPermissionsV2

<div class="padding-bottom--md">
<strong>useDeleteAllUsersPermissionsV2(): Object</strong>

<div class="padding-left--md">

Delete the all-users permissions



</div>







</div>





### useDeleteAppState

<div class="padding-bottom--md">
<strong>useDeleteAppState(): Object</strong>

<div class="padding-left--md">

Deletes app state



</div>







</div>





### useDeleteAppStates

<div class="padding-bottom--md">
<strong>useDeleteAppStates(): Object</strong>

<div class="padding-left--md">

Delete all app states



</div>







</div>





### useDeleteDocument

<div class="padding-bottom--md">
<strong>useDeleteDocument(): Object</strong>

<div class="padding-left--md">

Delete the document



</div>







</div>





### ~~useDeleteSettings~~

> ⚠️ **Deprecated**
> Use V2 instead.

<div class="padding-bottom--md">
<strong>useDeleteSettings(): Object</strong>

<div class="padding-left--md">

Delete the setting



</div>







</div>





### useDeleteSettingsV2

<div class="padding-bottom--md">
<strong>useDeleteSettingsV2(): Object</strong>

<div class="padding-left--md">

Delete the setting



</div>







</div>





### useDeleteUserAppState

<div class="padding-bottom--md">
<strong>useDeleteUserAppState(): Object</strong>

<div class="padding-left--md">

Delete user app state



</div>







</div>





### useDeleteUserAppStates

<div class="padding-bottom--md">
<strong>useDeleteUserAppStates(): Object</strong>

<div class="padding-left--md">

Delete all user app states



</div>







</div>





### useDocument

<div class="padding-bottom--md">
<strong>useDocument(params,options?): Object</strong>

<div class="padding-left--md">

Retrieve metadata and content for documents.



</div>


#### Parameters

| Name | Type | Description |
| --- | --- | --- |
|params<sup>*required</sup>|DocumentParams|Parameters for the client sdk call. |
|options|HookOptions|Additional options for the react hook. |








</div>





### useDocumentMetaData

<div class="padding-bottom--md">
<strong>useDocumentMetaData(params,options?): Object</strong>

<div class="padding-left--md">

Retrieve document metadata.



</div>


#### Parameters

| Name | Type | Description |
| --- | --- | --- |
|params<sup>*required</sup>|DocumentMetaDataParams|Parameters for the client sdk call. |
|options|HookOptions|Additional options for the react hook. |








</div>





### useDownloadDocument

<div class="padding-bottom--md">
<strong>useDownloadDocument(params,options?): Object</strong>

<div class="padding-left--md">

Download document content



</div>


#### Parameters

| Name | Type | Description |
| --- | --- | --- |
|params<sup>*required</sup>|DownloadDocumentParams|Parameters for the client sdk call. |
|options|HookOptions|Additional options for the react hook. |








</div>





### useDql

<div class="padding-bottom--md">
<strong>useDql(query,options?): [UseDqlResult](#usedqlresult)&lt;T&gt;</strong>

<div class="padding-left--md">

React hook for executing DQL queries.



</div>


#### Parameters

| Name | Type | Description |
| --- | --- | --- |
|query<sup>*required</sup>|[string](https://developer.mozilla.org/en-US/docs/Glossary/String) &#124; [DqlQueryParams](#dqlqueryparams)|Query and enrich options that will be executed. |
|options|[UseDqlOptions](#usedqloptions)|Additional options for the React hook. |




#### Returns

| Return type | Description |
|---|---|
|[UseDqlResult](#usedqlresult) | The result of the query, variables indicating the current state and functions to cancel or refetch the query.|





</div>





### ~~useDqlQuery~~

> ⚠️ **Deprecated**
> Use `useDql` hook instead.

<div class="padding-bottom--md">
<strong>useDqlQuery(params,options?): Object</strong>

<div class="padding-left--md">

Use this hook to execute a DQL query.



</div>


#### Parameters

| Name | Type | Description |
| --- | --- | --- |
|~~params~~<sup>**DEPRECATED**</sup><sup>*required</sup>|QueryParams|Query and enrich options that will be executed. |
|~~options~~<sup>**DEPRECATED**</sup>|HookOptions|Additional options for the react hook. |




#### Returns

| Description |
|---|
|The state of the query, including the result and functions to refetch or cancel the query.|





</div>





### ~~useEffectivePermissions~~

> ⚠️ **Deprecated**
> Use V2 instead.

<div class="padding-bottom--md">
<strong>useEffectivePermissions(params,options?): Object</strong>

<div class="padding-left--md">

Retrieve effective settings permissions for the calling user.



</div>


#### Parameters

| Name | Type | Description |
| --- | --- | --- |
|~~params~~<sup>**DEPRECATED**</sup><sup>*required</sup>|Omit&lt;Object &#124; "abortSignal"&gt;|Parameters for the client sdk call. |
|~~options~~<sup>**DEPRECATED**</sup>|HookOptions|Additional options for the react hook. |








</div>





### useEffectivePermissionsV2

<div class="padding-bottom--md">
<strong>useEffectivePermissionsV2(params,options?): Object</strong>

<div class="padding-left--md">

Retrieve effective settings permissions for the calling user.



</div>


#### Parameters

| Name | Type | Description |
| --- | --- | --- |
|params<sup>*required</sup>|Omit&lt;Object &#124; "abortSignal"&gt;|Parameters for the client sdk call. |
|options|HookOptions|Additional options for the react hook. |








</div>





### useListDocuments

<div class="padding-bottom--md">
<strong>useListDocuments(params,options?): Object</strong>

<div class="padding-left--md">

List all documents accessible to you.



</div>


#### Parameters

| Name | Type | Description |
| --- | --- | --- |
|params<sup>*required</sup>|undefined &#124; ListDocumentsParams|Parameters for the client sdk call. |
|options|HookOptions|Additional options for the react hook. |








</div>





### usePermissionsV2

<div class="padding-bottom--md">
<strong>usePermissionsV2(params,options?): Object</strong>

<div class="padding-left--md">

Load permissions.



</div>


#### Parameters

| Name | Type | Description |
| --- | --- | --- |
|params<sup>*required</sup>|PermissionsParamsV2|Parameters for the client sdk call. |
|options|HookOptions|Additional options for the react hook. |








</div>





### useSetAppState

<div class="padding-bottom--md">
<strong>useSetAppState(): Object</strong>

<div class="padding-left--md">

Updates app state



</div>







</div>





### useSetUserAppState

<div class="padding-bottom--md">
<strong>useSetUserAppState(): Object</strong>

<div class="padding-left--md">

Updates user app state



</div>







</div>





### ~~useSettings~~

> ⚠️ **Deprecated**
> Use V2 instead.

<div class="padding-bottom--md">
<strong>useSettings(params,options?): Object</strong>

<div class="padding-left--md">

Retrieve metadata and content for settings.



</div>


#### Parameters

| Name | Type | Description |
| --- | --- | --- |
|~~params~~<sup>**DEPRECATED**</sup><sup>*required</sup>|SettingsParams|Parameters for the client sdk call. |
|~~options~~<sup>**DEPRECATED**</sup>|HookOptions|Additional options for the react hook. |








</div>





### ~~useSettingsObjects~~

> ⚠️ **Deprecated**
> Use V2 instead.

<div class="padding-bottom--md">
<strong>useSettingsObjects(params,options?): Object</strong>

<div class="padding-left--md">

Retrieve metadata and content for settings.



</div>


#### Parameters

| Name | Type | Description |
| --- | --- | --- |
|~~params~~<sup>**DEPRECATED**</sup><sup>*required</sup>|SettingsObjectParams|Parameters for the client sdk call. |
|~~options~~<sup>**DEPRECATED**</sup>|HookOptions|Additional options for the react hook. |








</div>





### useSettingsObjectsV2

<div class="padding-bottom--md">
<strong>useSettingsObjectsV2(params,options?): Object</strong>

<div class="padding-left--md">

Retrieve metadata and content for settings.



</div>


#### Parameters

| Name | Type | Description |
| --- | --- | --- |
|params<sup>*required</sup>|SettingsObjectParamsV2|Parameters for the client sdk call. |
|options|HookOptions|Additional options for the react hook. |








</div>





### useSettingsV2

<div class="padding-bottom--md">
<strong>useSettingsV2(params,options?): Object</strong>

<div class="padding-left--md">

Retrieve metadata and content for settings.



</div>


#### Parameters

| Name | Type | Description |
| --- | --- | --- |
|params<sup>*required</sup>|SettingsParamsV2|Parameters for the client sdk call. |
|options|HookOptions|Additional options for the react hook. |








</div>





### useTransferOwnershipV2

<div class="padding-bottom--md">
<strong>useTransferOwnershipV2(): Object</strong>

<div class="padding-left--md">

Transfer ownership.



</div>







</div>





### useUpdateAccessorPermissionsV2

<div class="padding-bottom--md">
<strong>useUpdateAccessorPermissionsV2(): Object</strong>

<div class="padding-left--md">

Update accessor permissions.



</div>







</div>





### useUpdateAllUsersPermissionsV2

<div class="padding-bottom--md">
<strong>useUpdateAllUsersPermissionsV2(): Object</strong>

<div class="padding-left--md">

Update all-users permissions.



</div>







</div>





### useUpdateDocument

<div class="padding-bottom--md">
<strong>useUpdateDocument(): Object</strong>

<div class="padding-left--md">

Update metadata and content.



</div>







</div>





### useUpdateDocumentMetadata

<div class="padding-bottom--md">
<strong>useUpdateDocumentMetadata(): Object</strong>

<div class="padding-left--md">

Update document metadata



</div>







</div>





### ~~useUpdateSettings~~

> ⚠️ **Deprecated**
> Use V2 instead.

<div class="padding-bottom--md">
<strong>useUpdateSettings(): Object</strong>

<div class="padding-left--md">

Update metadata and content.



</div>







</div>





### useUpdateSettingsV2

<div class="padding-bottom--md">
<strong>useUpdateSettingsV2(): Object</strong>

<div class="padding-left--md">

Update metadata and content.



</div>







</div>





### useUserAppState

<div class="padding-bottom--md">
<strong>useUserAppState(params,options?): Object</strong>

<div class="padding-left--md">

Get user app state



</div>


#### Parameters

| Name | Type | Description |
| --- | --- | --- |
|params<sup>*required</sup>|UserAppStateParams|Parameters for the client sdk call. |
|options|HookOptions|Additional options for the react hook. |








</div>





### useUserAppStates

<div class="padding-bottom--md">
<strong>useUserAppStates(params,options?): Object</strong>

<div class="padding-left--md">

List user app states



</div>


#### Parameters

| Name | Type | Description |
| --- | --- | --- |
|params<sup>*required</sup>|UserAppStatesParams|Parameters for the client sdk call. |
|options|HookOptions|Additional options for the react hook. |








</div>



## Components



### DqlQueryParamsProvider

<div class="padding-bottom--md">


<div class="padding-left--md">

Provider component for Dql query params context to use with the `useDql` hook.



</div>


#### Parameters

| Name | Type |
| --- | --- |
|props<sup>*required</sup>|[DqlQueryParamsContextProps](#dqlqueryparamscontextprops)| 








</div>



## Variables

<div class="padding-left--md padding-bottom--md">
<strong>DqlQueryParamsContext</strong>: Context&lt;[DqlQueryParamsContextProps](#dqlqueryparamscontextprops)&gt;
<div class="padding-left--md">

Context storing Dql query params.

</div>



</div>

## Interfaces

### AnalyzerHookOptions



#### Properties

| Name | Type | Description |
| --- | --- | --- |
|~~autoFetch~~<sup>**DEPRECATED**</sup><sup>*required</sup>|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|If set to true, the hook will execute the query immediately. If set to false, the query only executes when calling refetch. |
|~~autoFetchOnUpdate~~<sup>**DEPRECATED**</sup><sup>*required</sup>|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|If set to true, the hook will execute the query on a component update. If set to false, it is not executed on update. |
|~~onError~~<sup>**DEPRECATED**</sup>|Object| |


### DqlQueryParams
Parameters for query execution.


#### Properties

| Name | Type | Description |
| --- | --- | --- |
|defaultSamplingRatio|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|In case not specified in the DQL string, the sampling ratio defined here is applied. Note that this is only applicable to log queries. |
|defaultScanLimitGbytes|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|Limit in gigabytes for the amount data that will be scanned during read. |
|defaultTimeframeEnd|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The query timeframe 'end' timestamp in ISO-8601 or RFC3339 format. If the timeframe 'start' parameter is missing, the whole timeframe is ignored. <em>Note that if a timeframe is specified within the query string (query) then it has precedence over this query request parameter.</em> |
|defaultTimeframeStart|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The query timeframe 'start' timestamp in ISO-8601 or RFC3339 format. If the timeframe 'end' parameter is missing, the whole timeframe is ignored. <em>Note that if a timeframe is specified within the query string (query) then it has precedence over this query request parameter.</em> |
|dtClientContext|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The dt-client-context header is an optional string parameter used for monitoring purposes. When included in a request, it helps retrieve information about the execution of the query. It shouldn't hold sensitive information. |
|enablePreview|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|Request preview results. If a preview is available within the requestTimeoutMilliseconds, then it will be returned as part of the response. |
|enforceQueryConsumptionLimit|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|(DEPRECATED use body parameter 'enforceQueryConsumptionLimit' instead) If set, query consumption limit will be enforced. |
|enrich|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|If set additional data will be available in the metadata section. |
|fetchTimeoutSeconds|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|The query will stop reading data after reaching the fetch-timeout. The query execution will continue, providing a partial result based on the read data. |
|filterSegments|[FilterSegments](#filtersegments)|Represents a collection of filter segments. |
|includeTypes|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|Parameter to exclude the type information from the query result. In case not specified, the type information will be included. |
|locale|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The query locale. If none specified, then a language/country neutral locale is chosen. The input values take the ISO-639 Language code with an optional ISO-3166 country code appended to it with an underscore. For instance, both values are valid 'en' or 'en_US'. |
|maxResultBytes|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|The maximum number of result bytes that this query will return. |
|maxResultRecords|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|The maximum number of result records that this query will return. |
|query<sup>*required</sup>|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The full query string. |
|queryOptions|[QueryOptions](#queryoptions)|Query options enhance query functionality for Dynatrace internal services. |
|requestTimeoutMilliseconds|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|The maximum time the response will be delayed to wait for a result. (This excludes the sending time and time spent in any services between the query-frontend and the client.) If the query finishes within the specified timeout, the query result is returned. Otherwise, the requestToken is returned, allowing polling for the result. |
|timezone|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The query timezone. If none is specified, UTC is used as fallback. The list of valid input values matches that of the IANA Time Zone Database (TZDB). It accepts values in their canonical names like 'Europe/Paris', the abbreviated version like CET or the UTC offset format like '+01:00' |


### DqlQueryParamsContextProps



#### Properties

| Name | Type | Description |
| --- | --- | --- |
|defaultSamplingRatio|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|In case not specified in the DQL string, the sampling ratio defined here is applied. Note that this is only applicable to log queries. |
|defaultScanLimitGbytes|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|Limit in gigabytes for the amount data that will be scanned during read. |
|defaultTimeframeEnd|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The query timeframe 'end' timestamp in ISO-8601 or RFC3339 format. If the timeframe 'start' parameter is missing, the whole timeframe is ignored. <em>Note that if a timeframe is specified within the query string (query) then it has precedence over this query request parameter.</em> |
|defaultTimeframeStart|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The query timeframe 'start' timestamp in ISO-8601 or RFC3339 format. If the timeframe 'end' parameter is missing, the whole timeframe is ignored. <em>Note that if a timeframe is specified within the query string (query) then it has precedence over this query request parameter.</em> |
|dtClientContext|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The dt-client-context header is an optional string parameter used for monitoring purposes. When included in a request, it helps retrieve information about the execution of the query. It shouldn't hold sensitive information. |
|enablePreview|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|Request preview results. If a preview is available within the requestTimeoutMilliseconds, then it will be returned as part of the response. |
|enforceQueryConsumptionLimit|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|(DEPRECATED use body parameter 'enforceQueryConsumptionLimit' instead) If set, query consumption limit will be enforced. |
|enrich|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|If set additional data will be available in the metadata section. |
|fetchTimeoutSeconds|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|The query will stop reading data after reaching the fetch-timeout. The query execution will continue, providing a partial result based on the read data. |
|filterSegments|[FilterSegments](#filtersegments)|Represents a collection of filter segments. |
|includeTypes|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|Parameter to exclude the type information from the query result. In case not specified, the type information will be included. |
|locale|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The query locale. If none specified, then a language/country neutral locale is chosen. The input values take the ISO-639 Language code with an optional ISO-3166 country code appended to it with an underscore. For instance, both values are valid 'en' or 'en_US'. |
|maxResultBytes|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|The maximum number of result bytes that this query will return. |
|maxResultRecords|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|The maximum number of result records that this query will return. |
|queryOptions|[QueryOptions](#queryoptions)|Query options enhance query functionality for Dynatrace internal services. |
|requestTimeoutMilliseconds|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|The maximum time the response will be delayed to wait for a result. (This excludes the sending time and time spent in any services between the query-frontend and the client.) If the query finishes within the specified timeout, the query result is returned. Otherwise, the requestToken is returned, allowing polling for the result. |
|timezone|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)|The query timezone. If none is specified, UTC is used as fallback. The list of valid input values matches that of the IANA Time Zone Database (TZDB). It accepts values in their canonical names like 'Europe/Paris', the abbreviated version like CET or the UTC offset format like '+01:00' |


### FilterSegment
A filter segment is identified by an ID. Each segment includes a list of variable definitions.


#### Properties

| Name | Type |
| --- | --- |
|id<sup>*required</sup>|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)| 
|variables|Array&lt;[FilterSegmentVariableDefinition](#filtersegmentvariabledefinition)&gt;| 


### FilterSegmentVariableDefinition
Defines a variable with a name and a list of values.


#### Properties

| Name | Type |
| --- | --- |
|name<sup>*required</sup>|[string](https://developer.mozilla.org/en-US/docs/Glossary/String)| 
|values<sup>*required</sup>|Array&lt;[string](https://developer.mozilla.org/en-US/docs/Glossary/String)&gt;| 


### FilterSegments
Represents a collection of filter segments.


### QueryOptions
Query options enhance query functionality for Dynatrace internal services.


### RefetchOptions
Options for the refetch functions returned by the useDql hook.


#### Properties

| Name | Type | Description |
| --- | --- | --- |
|cancelRefetch|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|<p>If set to <code>true</code>, a currently running request will be cancelled before a new request is made</p> <p>If set to <code>false</code>, no refetch will be made if there is already a request running.</p> <p>Defaults to <code>true</code>.</p>  |
|throwOnError|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)| |


### TypedQueryResult
Generic type that allows to type data returned from Grail.


#### Properties

| Name | Type | Description |
| --- | --- | --- |
|metadata<sup>*required</sup>|Metadata|Collects various bits of metadata information. |
|records<sup>*required</sup>|Array&lt;T&gt;| |
|types<sup>*required</sup>|Array&lt;RangedFieldTypes&gt;|The data types for the result records. |


### UseDqlOptions
Options for the useDql hook.


#### Properties

| Name | Type | Description |
| --- | --- | --- |
|enabled|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|Set this to <code>false</code> to disable this query from automatically running. Default: <code>true</code> |
|pollRate|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|The rate at which Grail query is polled |
|refetchInterval|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|Query will continuously refetch at this frequency in milliseconds |
|staleTime|[number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|The time in milliseconds after which data is considered stale. |


### UseDqlResult
Represents the possible results of the useDql hook.


#### Properties

| Name | Type | Description |
| --- | --- | --- |
|cancel<sup>*required</sup>|Object|Function to call to cancel to ongoing query |
|data|[TypedQueryResult](#typedqueryresult)&lt;T&gt;|The last successfully resolved data for the query. |
|error<sup>*required</sup>|null &#124; Error|The error object for the query, if an error was thrown. |
|forceRefetch<sup>*required</sup>|Object|Function to manually force refetch the query. Ignores stale time. |
|isError<sup>*required</sup>|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|<code>true</code> if the query attempt resulted in an error. |
|isFetched<sup>*required</sup>|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|<code>true</code> if the query has been fetched. |
|isFetching<sup>*required</sup>|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|<code>true</code> whenever the query is executing, which includes initial pending as well as background refetches. |
|isLoading<sup>*required</sup>|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|<code>true</code> whenever the first fetch for a query is in-flight. |
|isLoadingError<sup>*required</sup>|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|Will be <code>true</code> if the query failed while fetching for the first time. |
|isPending<sup>*required</sup>|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|<code>true</code> if there's no cached data and no query attempt was finished yet. |
|isRefetchError<sup>*required</sup>|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|<code>true</code> if the query failed while refetching. |
|isStale<sup>*required</sup>|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|<code>true</code> if the data in the cache is invalidated or if the data is older than the given <code>staleTime</code>. |
|isSuccess<sup>*required</sup>|[boolean](https://developer.mozilla.org/en-US/docs/Glossary/Boolean)|<code>true</code> if the query has received a response with no errors and is ready to display the data. |
|progress<sup>*required</sup>|undefined &#124; [number](https://developer.mozilla.org/en-US/docs/Glossary/Number)|The progress of the query from 0 to 100. |
|refetch<sup>*required</sup>|Object|Function to manually refetch the query. |
|state<sup>*required</sup>|"FAILED" &#124; "RUNNING" &#124; "NOT_STARTED" &#124; "SUCCEEDED" &#124; "RESULT_GONE" &#124; "CANCELLED"|Possible state of the query. |


