/**
 * @license
 * Copyright 2023-2025 Dynatrace LLC
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// packages/util/react-hooks/src/useAnalyzer.ts
import {
  analyzersClient
} from "@dynatrace-sdk/client-davis-analyzers";

// packages/util/react-hooks/src/react-async-hook/react-async-hook.ts
import {
  useCallback,
  useEffect,
  useLayoutEffect,
  useRef,
  useState
} from "react";
var useIsomorphicLayoutEffect = typeof window !== "undefined" && typeof window.document !== "undefined" && typeof window.document.createElement !== "undefined" ? useLayoutEffect : useEffect;
var useGetter = (t) => {
  const ref = useRef(t);
  useIsomorphicLayoutEffect(() => {
    ref.current = t;
  });
  return useCallback(() => ref.current, [ref]);
};
var InitialAsyncState = {
  status: "not-requested",
  loading: false,
  result: void 0,
  error: void 0
};
var InitialAsyncLoadingState = {
  status: "loading",
  loading: true,
  result: void 0,
  error: void 0
};
var defaultSetLoading = (_asyncState) => InitialAsyncLoadingState;
var defaultSetResult = (result, _asyncState) => ({
  status: "success",
  loading: false,
  result,
  error: void 0
});
var defaultSetError = (error, _asyncState) => ({
  status: "error",
  loading: false,
  result: void 0,
  error
});
var noop = () => {
};
var DefaultOptions = {
  initialState: (options) => options && options.executeOnMount ? InitialAsyncLoadingState : InitialAsyncState,
  executeOnMount: true,
  executeOnUpdate: true,
  setLoading: defaultSetLoading,
  setResult: defaultSetResult,
  setError: defaultSetError,
  onSuccess: noop,
  onError: noop
};
var normalizeOptions = (options) => ({
  ...DefaultOptions,
  ...options
});
var useAsyncState = (options) => {
  const [value, setValue] = useState(() => options.initialState(options));
  const reset = useCallback(() => setValue(options.initialState(options)), [setValue, options]);
  const setLoading = useCallback(() => setValue(options.setLoading(value)), [value, setValue]);
  const setResult = useCallback(
    (result) => setValue(options.setResult(result, value)),
    [value, setValue]
  );
  const setError = useCallback(
    (error) => setValue(options.setError(error, value)),
    [value, setValue]
  );
  const merge = useCallback(
    (state) => setValue({
      ...value,
      ...state
    }),
    [value, setValue]
  );
  return {
    value,
    set: setValue,
    merge,
    reset,
    setLoading,
    setResult,
    setError
  };
};
var useIsMounted = () => {
  const ref = useRef(false);
  useEffect(() => {
    ref.current = true;
    return () => {
      ref.current = false;
    };
  }, []);
  return () => ref.current;
};
var useCurrentPromise = () => {
  const ref = useRef(null);
  return {
    set: (promise) => ref.current = promise,
    get: () => ref.current,
    is: (promise) => ref.current === promise
  };
};
var useAsyncInternal = (asyncFunction, params, options) => {
  !params && (params = []);
  const normalizedOptions = normalizeOptions(options);
  const [currentParams, setCurrentParams] = useState(null);
  const AsyncState = useAsyncState(normalizedOptions);
  const isMounted = useIsMounted();
  const CurrentPromise = useCurrentPromise();
  const shouldHandlePromise = (p) => isMounted() && CurrentPromise.is(p);
  const executeAsyncOperation = (...args) => {
    const promise = (async () => asyncFunction(...args))();
    setCurrentParams(args);
    CurrentPromise.set(promise);
    AsyncState.setLoading();
    promise.then(
      (result) => {
        if (shouldHandlePromise(promise)) {
          AsyncState.setResult(result);
        }
        normalizedOptions.onSuccess(result, {
          isCurrent: () => CurrentPromise.is(promise)
        });
      },
      (error) => {
        if (shouldHandlePromise(promise)) {
          AsyncState.setError(error);
        }
        normalizedOptions.onError(error, {
          isCurrent: () => CurrentPromise.is(promise)
        });
      }
    );
    return promise;
  };
  const getLatestExecuteAsyncOperation = useGetter(executeAsyncOperation);
  const executeAsyncOperationMemo = useCallback(
    (...args) => {
      try {
        return getLatestExecuteAsyncOperation()(...args);
      } catch (e) {
        return Promise.resolve(void 0);
      }
    },
    [getLatestExecuteAsyncOperation]
  );
  const isMounting = !isMounted();
  useEffect(() => {
    const execute = () => getLatestExecuteAsyncOperation()(...params);
    isMounting && normalizedOptions.executeOnMount && execute();
    !isMounting && normalizedOptions.executeOnUpdate && execute();
  }, params);
  return {
    ...AsyncState.value,
    set: AsyncState.set,
    merge: AsyncState.merge,
    reset: AsyncState.reset,
    execute: executeAsyncOperationMemo,
    currentPromise: CurrentPromise.get(),
    currentParams
  };
};
function useAsync(asyncFunction, params, options) {
  return useAsyncInternal(asyncFunction, params, options);
}
var useAsyncAbortable = (asyncFunction, params, options) => {
  const abortControllerRef = useRef();
  const asyncFunctionWrapper = async (...p) => {
    if (abortControllerRef.current) {
      abortControllerRef.current.abort();
    }
    const abortController = new AbortController();
    abortControllerRef.current = abortController;
    try {
      return await asyncFunction(abortController.signal, ...p);
    } finally {
      if (abortControllerRef.current === abortController) {
        abortControllerRef.current = void 0;
      }
    }
  };
  return {
    ...useAsync(asyncFunctionWrapper, params, options),
    cancel: () => abortControllerRef?.current?.abort()
  };
};
var useAsyncCallback = (asyncFunction, options) => {
  return useAsyncInternal(
    asyncFunction,
    // Hacky but in such case we don't need the params,
    // because async function is only executed manually
    [],
    {
      ...options,
      executeOnMount: false,
      executeOnUpdate: false
    }
  );
};

// packages/util/react-hooks/src/util/delay-util.ts
var delay = (ms, value, signal) => new Promise((resolve, reject) => {
  const listener = () => {
    clearTimeout(timer);
    reject(value);
  };
  const timer = setTimeout(() => {
    signal?.removeEventListener("abort", listener);
    resolve(value);
  }, ms);
  signal?.addEventListener("abort", listener);
});

// packages/util/react-hooks/src/useAnalyzer.ts
var useAnalyzer = (params, options = { autoFetch: true, autoFetchOnUpdate: true }) => {
  const onError = typeof options.onError === "function" ? options.onError : () => void 0;
  const result = useAsyncAbortable(
    (signal) => doExecute(params, signal),
    [JSON.stringify(params)],
    {
      executeOnMount: options.autoFetch,
      executeOnUpdate: options.autoFetchOnUpdate,
      onError
    }
  );
  return getResult(result);
};
var doExecute = async (params, abortSignal) => {
  const executeResult = await analyzersClient.executeAnalyzer({ ...params, abortSignal });
  if (executeResult.result.executionStatus === "COMPLETED") {
    return executeResult.result;
  }
  let pollResult = null;
  while (executeResult.requestToken && (pollResult === null || pollResult.result.executionStatus === "RUNNING")) {
    try {
      await delay(1e3, null, abortSignal);
      pollResult = await analyzersClient.pollAnalyzerExecution({
        analyzerName: params.analyzerName,
        requestToken: executeResult.requestToken,
        abortSignal
      });
    } finally {
      await abortAnalyzer(abortSignal, params.analyzerName, executeResult.requestToken);
    }
  }
  return pollResult?.result;
};
var getResult = (result) => {
  let error = result.error;
  if (result.error?.name === "AbortError")
    error = void 0;
  return {
    data: result.result,
    error: result.error,
    errorDetails: error?.body?.error,
    isError: error ? true : false,
    isLoading: result.loading,
    isSuccess: !error && !result.loading && result.result,
    status: result.status,
    refetch: result.execute,
    cancel: result.cancel
  };
};
var abortAnalyzer = async function(signal, analyzerName, requestToken) {
  if (signal?.aborted) {
    await analyzersClient.cancelAnalyzerExecution({
      analyzerName,
      requestToken
    });
    return Promise.reject();
  }
};

// packages/util/react-hooks/src/useAppState.ts
import { stateClient } from "@dynatrace-sdk/client-state";
import { useCallback as useCallback2 } from "react";
function useAppState(params, options = { autoFetch: true, autoFetchOnUpdate: true }) {
  const onError = typeof options.onError === "function" ? options.onError : () => void 0;
  const result = useAsyncAbortable(
    (signal) => {
      return stateClient.getAppState({ ...params, abortSignal: signal });
    },
    [JSON.stringify(params)],
    {
      executeOnMount: options.autoFetch,
      executeOnUpdate: options.autoFetchOnUpdate,
      onError
    }
  );
  return getResult2(result);
}
function useAppStates(params, options = { autoFetch: true, autoFetchOnUpdate: true }) {
  const onError = typeof options.onError === "function" ? options.onError : () => void 0;
  const result = useAsyncAbortable(
    (signal) => stateClient.getAppStates({ ...params, abortSignal: signal }),
    [JSON.stringify(params)],
    {
      executeOnMount: options.autoFetch,
      executeOnUpdate: options.autoFetchOnUpdate,
      onError
    }
  );
  return getResult2(result);
}
function useUserAppState(params, options = { autoFetch: true, autoFetchOnUpdate: true }) {
  const onError = typeof options.onError === "function" ? options.onError : () => void 0;
  const result = useAsyncAbortable(
    (signal) => {
      return stateClient.getUserAppState({ ...params, abortSignal: signal });
    },
    [JSON.stringify(params)],
    {
      executeOnMount: options.autoFetch,
      executeOnUpdate: options.autoFetchOnUpdate,
      onError
    }
  );
  return getResult2(result);
}
function useUserAppStates(params, options = { autoFetch: true, autoFetchOnUpdate: true }) {
  const onError = typeof options.onError === "function" ? options.onError : () => void 0;
  const result = useAsyncAbortable(
    (signal) => stateClient.getUserAppStates({ ...params, abortSignal: signal }),
    [JSON.stringify(params)],
    {
      executeOnMount: options.autoFetch,
      executeOnUpdate: options.autoFetchOnUpdate,
      onError
    }
  );
  return getResult2(result);
}
function useDeleteAppState() {
  const execute = useCallback2(async (params) => {
    await result.execute(params);
  }, []);
  const result = useAsyncCallback((params) => stateClient.deleteAppState({ ...params }));
  return getCallbackResult(result, execute);
}
function useDeleteAppStates() {
  const result = useAsyncCallback(() => stateClient.deleteAppStates());
  return getCallbackResult(result, result.execute);
}
function useDeleteUserAppState() {
  const execute = useCallback2(async (params) => {
    await result.execute(params);
  }, []);
  const result = useAsyncCallback((params) => stateClient.deleteUserAppState({ ...params }));
  return getCallbackResult(result, execute);
}
function useDeleteUserAppStates() {
  const result = useAsyncCallback(() => stateClient.deleteUserAppStates());
  return getCallbackResult(result, result.execute);
}
function useSetAppState() {
  const execute = useCallback2(async (params) => {
    await result.execute(params);
  }, []);
  const result = useAsyncCallback(
    (params) => stateClient.setAppState({ ...params })
  );
  return getCallbackResult(result, execute);
}
function useSetUserAppState() {
  const execute = useCallback2(async (params) => {
    await result.execute(params);
  }, []);
  const result = useAsyncCallback(
    (params) => stateClient.setUserAppState({ ...params })
  );
  return getCallbackResult(result, execute);
}
function getCallbackResult(result, execute) {
  const error = result.error;
  return {
    execute,
    data: result.result,
    error: result.error,
    errorDetails: error?.body?.error,
    isError: error ? true : false,
    isSuccess: !error && !result.loading && result.result,
    isLoading: result.loading,
    status: result.status
  };
}
function getResult2(result) {
  const error = result.error;
  return {
    data: result.result,
    error: result.error,
    errorDetails: error?.body?.error,
    isError: error ? true : false,
    isLoading: result.loading,
    isSuccess: !error && !result.loading && result.result,
    status: result.status,
    refetch: result.execute,
    cancel: result.cancel
  };
}

// packages/util/react-hooks/src/useDocument.ts
import { documentsClient } from "@dynatrace-sdk/client-document";
import { useRef as useRef2 } from "react";
function useDocument(params, options = { autoFetch: true, autoFetchOnUpdate: true }) {
  const onError = typeof options.onError === "function" ? options.onError : () => void 0;
  const result = useAsyncAbortable(
    (signal) => documentsClient.getDocument({ ...params, abortSignal: signal }),
    [JSON.stringify(params)],
    {
      executeOnMount: options.autoFetch,
      executeOnUpdate: options.autoFetchOnUpdate,
      onError
    }
  );
  return getResult3(result);
}
function useDocumentMetaData(params, options = { autoFetch: true, autoFetchOnUpdate: true }) {
  const onError = typeof options.onError === "function" ? options.onError : () => void 0;
  const result = useAsyncAbortable(
    (signal) => documentsClient.getDocumentMetadata({
      ...params,
      abortSignal: signal
    }),
    [JSON.stringify(params)],
    {
      executeOnMount: options.autoFetch,
      executeOnUpdate: options.autoFetchOnUpdate,
      onError
    }
  );
  return getResult3(result);
}
function useListDocuments(params, options = { autoFetch: true, autoFetchOnUpdate: true }) {
  const onError = typeof options.onError === "function" ? options.onError : () => void 0;
  const result = useAsyncAbortable(
    async (signal) => documentsClient.listDocuments({ ...params, abortSignal: signal }),
    [JSON.stringify(params)],
    {
      executeOnMount: options.autoFetch,
      executeOnUpdate: options.autoFetchOnUpdate,
      onError
    }
  );
  return getResult3(result);
}
function useDownloadDocument(params, options = { autoFetch: true, autoFetchOnUpdate: true }) {
  const onError = typeof options.onError === "function" ? options.onError : () => void 0;
  const result = useAsyncAbortable(
    (signal) => documentsClient.downloadDocumentContent({ ...params, abortSignal: signal }),
    [JSON.stringify(params)],
    {
      executeOnMount: options.autoFetch,
      executeOnUpdate: options.autoFetchOnUpdate,
      onError
    }
  );
  return getResult3(result);
}
function useCreateDocument() {
  const executeParams = useRef2();
  const execute = async (params) => {
    executeParams.current = params;
    await result.execute();
  };
  const result = useAsyncCallback(() => {
    if (executeParams.current) {
      return documentsClient.createDocument({ ...executeParams.current });
    }
  });
  return getCallbackResult2(result, execute);
}
function useUpdateDocument() {
  const executeParams = useRef2();
  const execute = async (params) => {
    executeParams.current = params;
    await result.execute();
  };
  const result = useAsyncCallback(() => {
    if (executeParams.current) {
      return documentsClient.updateDocument({ ...executeParams.current });
    }
  });
  return getCallbackResult2(result, execute);
}
function useUpdateDocumentMetadata() {
  const executeParams = useRef2();
  const execute = async (params) => {
    executeParams.current = params;
    await result.execute();
  };
  const result = useAsyncCallback(() => {
    if (executeParams.current) {
      return documentsClient.updateDocumentMetadata({ ...executeParams.current });
    }
  });
  return getCallbackResult2(result, execute);
}
function useDeleteDocument() {
  const executeParams = useRef2();
  const execute = async (params) => {
    executeParams.current = params;
    await result.execute();
  };
  const result = useAsyncCallback(() => {
    if (executeParams.current) {
      return documentsClient.deleteDocument({ ...executeParams.current });
    }
  });
  return getCallbackResult2(result, execute);
}
function getCallbackResult2(result, execute) {
  const error = result.error;
  return {
    execute,
    data: result.result,
    error: result.error,
    errorDetails: error?.body?.error,
    isError: error ? true : false,
    isSuccess: !error && !result.loading && result.result,
    isLoading: result.loading,
    status: result.status
  };
}
function getResult3(result) {
  const error = result.error;
  return {
    data: result.result,
    error: result.error,
    errorDetails: error?.body?.error,
    isError: error ? true : false,
    isLoading: result.loading,
    isSuccess: !error && !result.loading && result.result,
    status: result.status,
    refetch: result.execute,
    cancel: result.cancel
  };
}

// packages/util/react-hooks/src/useDqlQuery.ts
import {
  queryExecutionClient,
  QueryState
} from "@dynatrace-sdk/client-query";
function useDqlQuery(params, options = { autoFetch: true, autoFetchOnUpdate: true }) {
  const onError = typeof options.onError === "function" ? options.onError : () => void 0;
  const result = useAsyncAbortable(
    (signal) => doQuery(params, signal),
    [JSON.stringify(params)],
    {
      executeOnMount: options.autoFetch,
      executeOnUpdate: options.autoFetchOnUpdate,
      onError
    }
  );
  let error = result.error;
  if (result.error?.name === "AbortError")
    error = void 0;
  return {
    data: result.result,
    error: error ? result.error : void 0,
    errorDetails: error?.body?.error,
    isError: error ? true : false,
    isLoading: result.loading,
    isSuccess: !error && !result.loading && !!result.result,
    status: result.status,
    refetch: result.execute,
    cancel: result.cancel
  };
}
var doQuery = async function(params, signal) {
  const executeResult = await queryExecutionClient.queryExecute({
    ...params,
    abortSignal: signal
  });
  if (!(executeResult.state === QueryState.Running || executeResult.state === QueryState.NotStarted || executeResult.result === void 0)) {
    return executeResult.result;
  }
  let queryResult = null;
  while (executeResult.requestToken && (queryResult === null || queryResult.state === QueryState.Running || queryResult.state === QueryState.NotStarted)) {
    try {
      await delay(1e3, null, signal);
    } finally {
      await abortQuery(signal, executeResult.requestToken);
    }
    try {
      queryResult = await queryExecutionClient.queryPoll({
        requestToken: executeResult.requestToken,
        enrich: params?.enrich,
        abortSignal: signal
      });
    } finally {
      await abortQuery(signal, executeResult.requestToken);
    }
  }
  return queryResult?.result;
};
var abortQuery = async function(signal, requestToken) {
  if (signal?.aborted) {
    await queryExecutionClient.queryCancel({
      requestToken
    });
    return Promise.reject();
  }
};

// packages/util/react-hooks/src/useAppFunction.ts
import { functions } from "@dynatrace-sdk/app-utils";
function useAppFunction({ name, data, responseType = "json" }, options = { autoFetch: true, autoFetchOnUpdate: true }) {
  const onError = typeof options.onError === "function" ? options.onError : () => void 0;
  const result = useAsyncAbortable(
    (signal) => functions.call(name, { data, abortSignal: signal }).then((res) => res[responseType]()),
    [JSON.stringify(data)],
    {
      executeOnMount: options.autoFetch,
      executeOnUpdate: options.autoFetchOnUpdate,
      onError
    }
  );
  return {
    data: result.result,
    error: result.error,
    isError: !!result.error,
    isLoading: result.loading,
    isSuccess: !result.error && !result.loading && !!result.result,
    status: result.status,
    refetch: result.execute,
    cancel: result.cancel
  };
}

// packages/util/react-hooks/src/useSettings.ts
import {
  appSettingsObjectsClient
} from "@dynatrace-sdk/client-app-settings";
import { useRef as useRef3 } from "react";
function useSettings(params, options = { autoFetch: true, autoFetchOnUpdate: true }) {
  const onError = typeof options.onError === "function" ? options.onError : () => void 0;
  const result = useAsyncAbortable(
    (signal) => {
      return appSettingsObjectsClient.getEffectiveAppSettingsValues({
        ...params,
        abortSignal: signal
      });
    },
    [JSON.stringify(params)],
    {
      executeOnMount: options.autoFetch,
      executeOnUpdate: options.autoFetchOnUpdate,
      onError
    }
  );
  return getResult4(result);
}
function useSettingsObjects(params, options = { autoFetch: true, autoFetchOnUpdate: true }) {
  const onError = typeof options.onError === "function" ? options.onError : () => void 0;
  const result = useAsyncAbortable(
    (signal) => {
      return appSettingsObjectsClient.getAppSettingsObjects({
        ...params,
        abortSignal: signal
      });
    },
    [JSON.stringify(params)],
    {
      executeOnMount: options.autoFetch,
      executeOnUpdate: options.autoFetchOnUpdate,
      onError
    }
  );
  return getResult4(result);
}
function useEffectivePermissions(params, options = { autoFetch: true, autoFetchOnUpdate: true }) {
  const onError = typeof options.onError === "function" ? options.onError : () => void 0;
  const result = useAsyncAbortable(
    (signal) => {
      return appSettingsObjectsClient.resolveEffectivePermissions({
        ...params,
        abortSignal: signal
      });
    },
    [JSON.stringify(params)],
    {
      executeOnMount: options.autoFetch,
      executeOnUpdate: options.autoFetchOnUpdate,
      onError
    }
  );
  return getResult4(result);
}
function useDeleteSettings() {
  const executeParams = useRef3();
  const execute = async (params) => {
    executeParams.current = params;
    await result.execute();
  };
  const result = useAsyncCallback(() => {
    if (executeParams.current) {
      return appSettingsObjectsClient.deleteAppSettingsObjectByObjectId({
        ...executeParams.current
      });
    }
  });
  return getCallbackResult3(result, execute);
}
function useUpdateSettings() {
  const executeParams = useRef3();
  const execute = async (params) => {
    executeParams.current = params;
    await result.execute();
  };
  const result = useAsyncCallback(() => {
    if (executeParams.current) {
      return appSettingsObjectsClient.putAppSettingsObjectByObjectId({ ...executeParams.current });
    }
  });
  return getCallbackResult3(result, execute);
}
function useCreateSettings() {
  const executeParams = useRef3();
  const execute = async (params) => {
    executeParams.current = params;
    await result.execute();
  };
  const result = useAsyncCallback(() => {
    if (executeParams.current) {
      return appSettingsObjectsClient.postAppSettingsObject({ ...executeParams.current });
    }
  });
  return getCallbackResult3(result, execute);
}
function getCallbackResult3(result, execute) {
  const error = result.error;
  return {
    execute,
    data: result.result,
    error: error ? result.error : void 0,
    errorDetails: error?.body?.error,
    isError: error ? true : false,
    isSuccess: !error && !result.loading && result.result,
    isLoading: result.loading,
    status: result.status
  };
}
function getResult4(result) {
  const error = result.error;
  return {
    data: result.result,
    error: error ? result.error : void 0,
    errorDetails: error?.body?.error,
    isError: error ? true : false,
    isLoading: result.loading,
    isSuccess: !error && !result.loading && result.result,
    status: result.status,
    refetch: result.execute,
    cancel: result.cancel
  };
}

// packages/util/react-hooks/src/useSettingsV2.ts
import { appSettingsObjectsClient as appSettingsObjectsClient2 } from "@dynatrace-sdk/client-app-settings-v2";
import { useRef as useRef4 } from "react";
function useSettingsV2(params, options = { autoFetch: true, autoFetchOnUpdate: true }) {
  const onError = typeof options.onError === "function" ? options.onError : () => void 0;
  const result = useAsyncAbortable(
    (signal) => {
      return appSettingsObjectsClient2.getEffectiveAppSettingsValues({
        ...params,
        abortSignal: signal
      });
    },
    [JSON.stringify(params)],
    {
      executeOnMount: options.autoFetch,
      executeOnUpdate: options.autoFetchOnUpdate,
      onError
    }
  );
  return getResult5(result);
}
function useSettingsObjectsV2(params, options = { autoFetch: true, autoFetchOnUpdate: true }) {
  const onError = typeof options.onError === "function" ? options.onError : () => void 0;
  const result = useAsyncAbortable(
    (signal) => {
      return appSettingsObjectsClient2.getAppSettingsObjects({
        ...params,
        abortSignal: signal
      });
    },
    [JSON.stringify(params)],
    {
      executeOnMount: options.autoFetch,
      executeOnUpdate: options.autoFetchOnUpdate,
      onError
    }
  );
  return getResult5(result);
}
function useEffectivePermissionsV2(params, options = { autoFetch: true, autoFetchOnUpdate: true }) {
  const onError = typeof options.onError === "function" ? options.onError : () => void 0;
  const result = useAsyncAbortable(
    (signal) => {
      return appSettingsObjectsClient2.resolveEffectivePermissions({
        ...params,
        abortSignal: signal
      });
    },
    [JSON.stringify(params)],
    {
      executeOnMount: options.autoFetch,
      executeOnUpdate: options.autoFetchOnUpdate,
      onError
    }
  );
  return getResult5(result);
}
function useDeleteSettingsV2() {
  const executeParams = useRef4();
  const execute = async (params) => {
    executeParams.current = params;
    await result.execute();
  };
  const result = useAsyncCallback(() => {
    if (executeParams.current) {
      return appSettingsObjectsClient2.deleteAppSettingsObjectByObjectId({
        ...executeParams.current
      });
    }
  });
  return getCallbackResult4(result, execute);
}
function useUpdateSettingsV2() {
  const executeParams = useRef4();
  const execute = async (params) => {
    executeParams.current = params;
    await result.execute();
  };
  const result = useAsyncCallback(() => {
    if (executeParams.current) {
      return appSettingsObjectsClient2.putAppSettingsObjectByObjectId({
        ...executeParams.current
      });
    }
  });
  return getCallbackResult4(result, execute);
}
function useCreateSettingsV2() {
  const executeParams = useRef4();
  const execute = async (params) => {
    executeParams.current = params;
    await result.execute();
  };
  const result = useAsyncCallback(() => {
    if (executeParams.current) {
      return appSettingsObjectsClient2.postAppSettingsObject({
        ...executeParams.current
      });
    }
  });
  return getCallbackResult4(result, execute);
}
function useTransferOwnershipV2() {
  const executeParams = useRef4();
  const execute = async (params) => {
    executeParams.current = params;
    await result.execute();
  };
  const result = useAsyncCallback(() => {
    if (executeParams.current) {
      return appSettingsObjectsClient2.postAppSettingsOwnershipByObjectId({
        ...executeParams.current
      });
    }
  });
  return getCallbackResult4(result, execute);
}
function useAccessorPermissionsV2(params, options = { autoFetch: true, autoFetchOnUpdate: true }) {
  const onError = typeof options.onError === "function" ? options.onError : () => void 0;
  const result = useAsyncAbortable(
    (signal) => {
      return appSettingsObjectsClient2.getAppSettingsPermissionByObjectIdAndAccessorId({
        ...params,
        abortSignal: signal
      });
    },
    [JSON.stringify(params)],
    {
      executeOnMount: options.autoFetch,
      executeOnUpdate: options.autoFetchOnUpdate,
      onError
    }
  );
  return getResult5(result);
}
function useUpdateAccessorPermissionsV2() {
  const executeParams = useRef4();
  const execute = async (params) => {
    executeParams.current = params;
    await result.execute();
  };
  const result = useAsyncCallback(() => {
    if (executeParams.current) {
      return appSettingsObjectsClient2.putAppSettingsPermissionByObjectIdAndAccessorId({
        ...executeParams.current
      });
    }
  });
  return getCallbackResult4(result, execute);
}
function useDeleteAccessorPermissionsV2() {
  const executeParams = useRef4();
  const execute = async (params) => {
    executeParams.current = params;
    await result.execute();
  };
  const result = useAsyncCallback(() => {
    if (executeParams.current) {
      return appSettingsObjectsClient2.deleteAppSettingsPermissionByObjectIdAndAccessorId({
        ...executeParams.current
      });
    }
  });
  return getCallbackResult4(result, execute);
}
function usePermissionsV2(params, options = { autoFetch: true, autoFetchOnUpdate: true }) {
  const onError = typeof options.onError === "function" ? options.onError : () => void 0;
  const result = useAsyncAbortable(
    (signal) => {
      return appSettingsObjectsClient2.getAppSettingsPermissionsByObjectId({
        ...params,
        abortSignal: signal
      });
    },
    [JSON.stringify(params)],
    {
      executeOnMount: options.autoFetch,
      executeOnUpdate: options.autoFetchOnUpdate,
      onError
    }
  );
  return getResult5(result);
}
function useCreatePermissionsV2() {
  const executeParams = useRef4();
  const execute = async (params) => {
    executeParams.current = params;
    await result.execute();
  };
  const result = useAsyncCallback(() => {
    if (executeParams.current) {
      return appSettingsObjectsClient2.postAppSettingsPermissionByObjectId({
        ...executeParams.current
      });
    }
  });
  return getCallbackResult4(result, execute);
}
function useAllUsersPermissionsV2(params, options = { autoFetch: true, autoFetchOnUpdate: true }) {
  const onError = typeof options.onError === "function" ? options.onError : () => void 0;
  const result = useAsyncAbortable(
    (signal) => {
      return appSettingsObjectsClient2.getAppSettingsAllUsersPermissionByObjectId({
        ...params,
        abortSignal: signal
      });
    },
    [JSON.stringify(params)],
    {
      executeOnMount: options.autoFetch,
      executeOnUpdate: options.autoFetchOnUpdate,
      onError
    }
  );
  return getResult5(result);
}
function useUpdateAllUsersPermissionsV2() {
  const executeParams = useRef4();
  const execute = async (params) => {
    executeParams.current = params;
    await result.execute();
  };
  const result = useAsyncCallback(() => {
    if (executeParams.current) {
      return appSettingsObjectsClient2.putAppSettingsAllUsersPermissionByObjectId({
        ...executeParams.current
      });
    }
  });
  return getCallbackResult4(result, execute);
}
function useDeleteAllUsersPermissionsV2() {
  const executeParams = useRef4();
  const execute = async (params) => {
    executeParams.current = params;
    await result.execute();
  };
  const result = useAsyncCallback(() => {
    if (executeParams.current) {
      return appSettingsObjectsClient2.deleteAppSettingsAllUsersPermissionByObjectId({
        ...executeParams.current
      });
    }
  });
  return getCallbackResult4(result, execute);
}
function getCallbackResult4(result, execute) {
  const error = result.error;
  return {
    execute,
    data: result.result,
    error: error ? result.error : void 0,
    errorDetails: error?.body?.error,
    isError: error ? true : false,
    isSuccess: !error && !result.loading && result.result,
    isLoading: result.loading,
    status: result.status
  };
}
function getResult5(result) {
  const error = result.error;
  return {
    data: result.result,
    error: error ? result.error : void 0,
    errorDetails: error?.body?.error,
    isError: error ? true : false,
    isLoading: result.loading,
    isSuccess: !error && !result.loading && result.result,
    status: result.status,
    refetch: result.execute,
    cancel: result.cancel
  };
}

// packages/util/react-hooks/src/use-dql/useDql.ts
import { useQuery } from "@tanstack/react-query";
import { useCallback as useCallback3, useContext, useEffect as useEffect2, useMemo, useState as useState2 } from "react";

// packages/util/react-hooks/src/use-dql/query.ts
import { QueryExecutionClient } from "@dynatrace-sdk/client-query";
import { httpClient } from "@dynatrace-sdk/http-client";
var queryExecutionClient2 = new QueryExecutionClient(httpClient, { transformDates: false });
async function executeQuery(params, signal, controller) {
  try {
    return await queryExecutionClient2.queryExecute({
      body: params,
      abortSignal: signal,
      ...params
    });
  } catch (error) {
    controller?.abort();
    throw error;
  }
}
async function pollQuery(requestToken, params, controller) {
  try {
    return await queryExecutionClient2.queryPoll({
      requestToken,
      ...params
    });
  } catch (error) {
    controller?.abort();
    throw error;
  }
}
async function cancelQuery(requestToken, params, controller) {
  controller?.abort();
  return await queryExecutionClient2.queryCancel({
    requestToken,
    ...params
  });
}

// packages/util/react-hooks/src/use-dql/utils.ts
import { isClientRequestError } from "@dynatrace-sdk/client-query";
var delay2 = (ms, value, signal) => new Promise((resolve, reject) => {
  const listener = () => {
    clearTimeout(timer);
    reject(value);
  };
  const timer = setTimeout(() => {
    signal?.removeEventListener("abort", listener);
    resolve(value);
  }, ms);
  signal?.addEventListener("abort", listener);
});
function isTooManyRequestsError(error) {
  return isClientRequestError(error) && error.response.status === 429;
}
function isString(val) {
  return typeof val === "string" || val instanceof String;
}
async function promiseStatus(promise) {
  const referenceObject = { name: "Victor" };
  const value = await Promise.race([promise, referenceObject]);
  return value === referenceObject ? "pending" : "fulfilled";
}

// packages/util/react-hooks/src/use-dql/context.ts
import { createContext, createElement } from "react";
var DqlQueryParamsContext = createContext({});
function DqlQueryParamsProvider(props) {
  const { children, ...params } = props;
  return createElement(
    DqlQueryParamsContext.Provider,
    {
      value: params
    },
    children
  );
}

// packages/util/react-hooks/src/use-dql/query-client.ts
import { QueryClient } from "@tanstack/react-query";
var defaultQueryOptions = {
  staleTime: 6e4,
  pollRate: 1e3,
  enabled: true
};
function getDefaultQueryClientConfig() {
  return {
    defaultOptions: {
      queries: {
        refetchOnMount: false,
        refetchOnWindowFocus: false,
        refetchOnReconnect: false,
        refetchInterval: false,
        staleTime: 6e4,
        retry: false
      }
    }
  };
}
var queryClient = new QueryClient(getDefaultQueryClientConfig());

// packages/util/react-hooks/src/use-dql/useDql.ts
var IS_POLLING_DONE_INTERVAL = 500;
var IS_POLLING_DONE_TIMEOUT = 5e3;
function getQueryOptions(opts, ctxOpts = {}) {
  return {
    ...defaultQueryOptions,
    ...ctxOpts,
    ...opts
  };
}
function normalizeQueryParams(query, ctxParams) {
  const queryString = isString(query) ? query : query.query;
  const params = isString(query) ? {} : query;
  return {
    ...ctxParams,
    ...params,
    query: queryString
  };
}
function getQueryKey(queryParams) {
  const { query, ...params } = queryParams;
  return [query, JSON.stringify(params)];
}
function getQueryClientOptions(opts) {
  return {
    staleTime: opts.staleTime,
    refetchInterval: opts.refetchInterval,
    enabled: opts.enabled
  };
}
function useDql(query, options = {}) {
  const queryParamsContext = useContext(DqlQueryParamsContext);
  const [progress, setProgress] = useState2(0);
  const [state, setState] = useState2("NOT_STARTED");
  const [data, setData] = useState2(void 0);
  const q = useMemo(
    () => normalizeQueryParams(query, queryParamsContext),
    [query, queryParamsContext]
  );
  const queryOpts = useMemo(() => getQueryOptions(options), [options]);
  const queryKey = useMemo(() => getQueryKey(q), [q]);
  const queryClientOpts = useMemo(() => getQueryClientOptions(queryOpts), [queryOpts]);
  const cancel = useCallback3(async () => {
    await queryClient.cancelQueries({ queryKey });
  }, [queryClient, queryKey]);
  const queryResult = useQuery(
    {
      // eslint-disable-next-line @tanstack/query/exhaustive-deps
      queryKey,
      ...queryClientOpts,
      queryFn: async ({ signal }) => {
        setState("NOT_STARTED");
        setProgress(0);
        const removeHandlersController = new AbortController();
        const cancelExecuteController = new AbortController();
        let executeQueryResponse = null;
        let timeout = 1;
        while (!executeQueryResponse?.result && !executeQueryResponse?.requestToken) {
          try {
            executeQueryResponse = await executeQuery(q, signal, removeHandlersController);
          } catch (error) {
            if (isTooManyRequestsError(error)) {
              await delay2(Math.min(timeout++ ** 2, 30) * 1e3, void 0, signal);
            } else {
              throw error;
            }
          }
        }
        if (executeQueryResponse?.requestToken) {
          signal.addEventListener(
            "abort",
            async () => {
              const response = await cancelQuery(
                executeQueryResponse.requestToken,
                q,
                removeHandlersController
              );
              setProgress(response?.progress ?? 0);
              setState(response?.state ?? "CANCELLED");
            },
            { signal: cancelExecuteController.signal }
          );
          addEventListener(
            "beforeunload",
            async () => {
              if (queryClient.isFetching({ queryKey })) {
                removeHandlersController.abort();
                navigator.sendBeacon(
                  `/platform/storage/query/v1/query:cancel?request-token=${encodeURIComponent(
                    executeQueryResponse.requestToken
                  )}`
                );
              }
            },
            { signal: removeHandlersController.signal }
          );
          addEventListener(
            "visibilitychange",
            async () => {
              if (document.hidden && queryClient.isFetching({ queryKey })) {
                await cancel();
              }
            },
            { signal: removeHandlersController.signal }
          );
        }
        if (executeQueryResponse?.progress && executeQueryResponse?.progress !== progress) {
          setProgress(executeQueryResponse.progress ?? 0);
        }
        if (executeQueryResponse?.state && executeQueryResponse?.state !== state) {
          setState(executeQueryResponse.state);
        }
        if (!(executeQueryResponse?.state === "RUNNING" || executeQueryResponse?.state === "NOT_STARTED" || executeQueryResponse?.result === void 0)) {
          return executeQueryResponse?.result;
        }
        let pollQueryResponse = null;
        while (!signal.aborted && executeQueryResponse.requestToken && (pollQueryResponse === null || pollQueryResponse.state === "RUNNING" || pollQueryResponse.state === "NOT_STARTED")) {
          await delay2(queryOpts.pollRate, void 0, signal);
          const pollPromise = pollQuery(
            executeQueryResponse.requestToken,
            q,
            removeHandlersController
          );
          if (!cancelExecuteController.signal.aborted) {
            cancelExecuteController.abort();
            signal.addEventListener("abort", async () => {
              setProgress(0);
              setState("CANCELLED");
              const isQueryCompleted = await new Promise((res) => {
                const interval = setInterval(async () => {
                  if (pollQueryResponse?.state === "SUCCEEDED") {
                    clearInterval(interval);
                    clearTimeout(timeout2);
                    res(true);
                  }
                  const isPollingFinished = await promiseStatus(pollPromise) === "fulfilled";
                  if (isPollingFinished && pollQueryResponse?.state !== "SUCCEEDED") {
                    clearInterval(interval);
                    clearTimeout(timeout2);
                    res(false);
                  }
                }, IS_POLLING_DONE_INTERVAL);
                const timeout2 = setTimeout(() => {
                  clearInterval(interval);
                  res(false);
                }, IS_POLLING_DONE_TIMEOUT);
              });
              if (!isQueryCompleted) {
                await cancelQuery(executeQueryResponse.requestToken, q, removeHandlersController);
              }
            });
          }
          pollQueryResponse = await pollPromise;
          const pollQueryResult = pollQueryResponse.result;
          if (q.enablePreview && pollQueryResult && pollQueryResponse.state !== "SUCCEEDED") {
            setData(pollQueryResult);
          }
          if (!signal.aborted && pollQueryResponse?.progress) {
            setProgress(pollQueryResponse.progress ?? 0);
          }
          if (!signal.aborted && pollQueryResponse?.state) {
            setState(pollQueryResponse.state);
          }
        }
        removeHandlersController.abort();
        return pollQueryResponse?.result;
      }
    },
    queryClient
  );
  const refetch = useCallback3(
    async (options2) => {
      if ((queryResult.isStale || !queryOpts.enabled) && !queryResult.isFetching) {
        return await forceRefetch(options2);
      }
      const baseReturnValue2 = {
        state,
        progress,
        cancel,
        data,
        error: queryResult.error,
        isSuccess: queryResult.isSuccess,
        isError: queryResult.isError,
        isPending: queryResult.isPending,
        isLoading: queryResult.isLoading,
        isLoadingError: queryResult.isLoadingError,
        isRefetchError: queryResult.isRefetchError,
        isFetching: queryResult.isFetching,
        isFetched: queryResult.isFetched,
        isStale: queryResult.isStale,
        refetch,
        forceRefetch
      };
      return baseReturnValue2;
    },
    [queryResult.isStale, queryOpts.enabled, queryResult.isFetching, queryResult.refetch]
  );
  const forceRefetch = useCallback3(
    async (options2) => {
      const refetchResult = await queryResult.refetch(options2);
      const baseReturnValue2 = {
        state,
        progress,
        cancel,
        data,
        error: refetchResult.error,
        isSuccess: refetchResult.isSuccess,
        isError: refetchResult.isError,
        isPending: refetchResult.isPending,
        isLoading: refetchResult.isLoading,
        isLoadingError: refetchResult.isLoadingError,
        isRefetchError: refetchResult.isRefetchError,
        isFetching: refetchResult.isFetching,
        isFetched: refetchResult.isFetched,
        isStale: refetchResult.isStale,
        refetch,
        forceRefetch
      };
      return baseReturnValue2;
    },
    [queryResult.refetch]
  );
  useEffect2(() => {
    if (queryResult.data)
      setData(queryResult.data);
  }, [queryResult.data]);
  const baseReturnValue = {
    state,
    progress,
    cancel,
    data,
    error: queryResult.error,
    isSuccess: queryResult.isSuccess,
    isError: queryResult.isError,
    isPending: queryResult.isPending,
    isLoading: queryResult.isLoading,
    isLoadingError: queryResult.isLoadingError,
    isRefetchError: queryResult.isRefetchError,
    isFetching: queryResult.isFetching,
    isFetched: queryResult.isFetched,
    isStale: queryResult.isStale,
    refetch,
    forceRefetch
  };
  return baseReturnValue;
}
export {
  DqlQueryParamsContext,
  DqlQueryParamsProvider,
  useAccessorPermissionsV2,
  useAllUsersPermissionsV2,
  useAnalyzer,
  useAppFunction,
  useAppState,
  useAppStates,
  useCreateDocument,
  useCreatePermissionsV2,
  useCreateSettings,
  useCreateSettingsV2,
  useDeleteAccessorPermissionsV2,
  useDeleteAllUsersPermissionsV2,
  useDeleteAppState,
  useDeleteAppStates,
  useDeleteDocument,
  useDeleteSettings,
  useDeleteSettingsV2,
  useDeleteUserAppState,
  useDeleteUserAppStates,
  useDocument,
  useDocumentMetaData,
  useDownloadDocument,
  useDql,
  useDqlQuery,
  useEffectivePermissions,
  useEffectivePermissionsV2,
  useListDocuments,
  usePermissionsV2,
  useSetAppState,
  useSetUserAppState,
  useSettings,
  useSettingsObjects,
  useSettingsObjectsV2,
  useSettingsV2,
  useTransferOwnershipV2,
  useUpdateAccessorPermissionsV2,
  useUpdateAllUsersPermissionsV2,
  useUpdateDocument,
  useUpdateDocumentMetadata,
  useUpdateSettings,
  useUpdateSettingsV2,
  useUserAppState,
  useUserAppStates
};
