import { AbortSignal, Binary, HttpClient } from '@dynatrace-sdk/http-client';
import { AnalyzerCancelResult } from '../models/analyzer-cancel-result';
import { AnalyzerDefinitionDetails } from '../models/analyzer-definition-details';
import { AnalyzerExecuteResult } from '../models/analyzer-execute-result';
import { AnalyzerInput } from '../models/analyzer-input';
import { AnalyzerPollResult } from '../models/analyzer-poll-result';
import { AnalyzerQueryResult } from '../models/analyzer-query-result';
import { AnalyzerValidationResult } from '../models/analyzer-validation-result';
import { ApiClientOptions } from './api-client-options';
export declare class AnalyzersClient {
    private httpClient;
    private shouldTransformDates;
    /**
     * @param {HttpClient} httpClientImplementation - You can provide custom http client as a parameter to constructor of a AnalyzersClient. Custom http client should implement {HttpClient} interface.
     * @param {ApiClientOptions} [options] - Optional configuration for SDK Client.
     * @example
     * const analyzersClientNodeJs = new AnalyzersClient(yourCustomImplementation);
     */
    constructor(httpClientImplementation: HttpClient, options?: ApiClientOptions);
    /**
     * Query all available analyzer definitions.
     *
     * **Required scope:** davis:analyzers:read<br/><br/>The analyzer definition contains meta-information about an analyzer, including its input and output structure:
     * * The `input` definition specifies which fields the analyzer accepts, some of which may be required.
     * * The `output` definition specifies how the returned analyzer result is structured.
     *
     * @returns {Promise<AnalyzerQueryResult>} All available analyzer definitions.
     *
     * @example <caption>Code example</caption>
     * import { analyzersClient } from "@dynatrace-sdk/client-davis-analyzers";
     *
     * const data = await analyzersClient.queryAnalyzers();
     *
     * @throws {AnalyzerErrorEnvelopeError} Insufficient permissions. | Client error. | Internal server error.
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    queryAnalyzers(config?: {
        /**
         * The cursor for the page of results. Find it in the `nextPageKey` field of the previous response.
         *
         * If the `page-key` parameter is used, no other query parameters must be provided.
         *
         * If not provided, the first page is always returned.
         */ pageKey?: string;
        /**
         * The amount of returned analyzer definitions in a single response payload.
         * By default, 50 is used. The maximum is 200.
         */ pageSize?: number;
        /**
         * Use the `filter` parameter to only fetch a subset of analyzer definitions.
         *
         * When using the operators `=`and `!=`, filtering is **case sensitive**.
         *
         * The following operators are supported:
         * * `=`
         * * `!=`
         * * `contains`
         * * `starts-with`
         * * `ends-with`
         * * `and`
         * * `or`
         *
         * Parenthesis and the `not` operator are currently not supported.
         *
         * The following fields are valid filtering parameters:
         * * `baseAnalyzer`
         * * `description`
         * * `displayName`
         * * `labels`
         * * `name`
         * * `type`
         *
         * Any other field will result in a HTTP 400 response.
         */ filter?: string;
        /**
         * Specify which meta-information to fetch in addition to the default fields `name`, `displayName` and `description`.
         *
         * Valid values are:
         * * `analyzerCall`
         * * `baseAnalyzer`
         * * `category`
         * * `input`
         * * `labels`
         * * `output`
         * * `type`
         *
         * Any other field will result in a HTTP 400 response.
         */ addFields?: string;
        abortSignal?: AbortSignal;
    }): Promise<AnalyzerQueryResult>;
    /**
     * Get the analyzer definition for an analyzer.
     *
     * **Required scope:** davis:analyzers:read<br/><br/>All available meta-information for the analyzer is returned.
     *
     * @returns {Promise<AnalyzerDefinitionDetails>} The definition of the requested analyzer.
     *
     * @example <caption>Code example</caption>
     * import { analyzersClient } from "@dynatrace-sdk/client-davis-analyzers";
     *
     * const data = await analyzersClient.getAnalyzer({
     *   analyzerName: "...",
     * });
     *
     * @throws {AnalyzerErrorEnvelopeError} Invalid or malformed request. | Insufficient permissions. | No analyzer available with the requested name. | Client error. | Internal server error.
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    getAnalyzer(config: {
        /** The name of the analyzer. */ analyzerName: string;
        abortSignal?: AbortSignal;
    }): Promise<AnalyzerDefinitionDetails>;
    /**
     * Get the documentation for an analyzer.
     *
     * **Required scope:** davis:analyzers:read<br/><br/>A documentation may provide further information and context for a given analyzer.
     * Not all analyzers provide documentation.
     *
     * The documentation is returned as Markdown.
     *
     * @returns {Promise<Binary>} The documentation of the requested analyzer.
     *
     * @example <caption>Code example</caption>
     * import { analyzersClient } from "@dynatrace-sdk/client-davis-analyzers";
     *
     * const data = await analyzersClient.getAnalyzerDocumentation(
     *   { analyzerName: "..." },
     * );
     *
     * @throws {AnalyzerErrorEnvelopeError} Invalid or malformed request. | Insufficient permissions. | No analyzer available with the requested name. | Client error. | Internal server error.
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    getAnalyzerDocumentation(config: {
        /** The name of the analyzer. */ analyzerName: string;
        abortSignal?: AbortSignal;
    }): Promise<Binary>;
    /**
     * Get the JSON schema for an analyzer input.
     *
     * **Required scope:** davis:analyzers:read<br/><br/>The [JSON schema](https://json-schema.org/) defines a standardized declaration of the input structure,
     * providing well-defined input documentation that is both human- and machine-readable.
     *
     * @returns {Promise<Record<string, any>>} The [JSON input schema](https://json-schema.org/) of the requested analyzer.
     *
     * @example <caption>Code example</caption>
     * import { analyzersClient } from "@dynatrace-sdk/client-davis-analyzers";
     *
     * const data = await analyzersClient.getJsonSchemaForInput({
     *   analyzerName: "...",
     * });
     *
     * @throws {AnalyzerErrorEnvelopeError} Invalid or malformed request. | Insufficient permissions. | No analyzer available with the requested name. | Client error. | Internal server error.
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    getJsonSchemaForInput(config: {
        /** The name of the analyzer. */ analyzerName: string;
        abortSignal?: AbortSignal;
    }): Promise<Record<string, any>>;
    /**
     * Get the JSON schema for an analyzer result.
     *
     * **Required scope:** davis:analyzers:read<br/><br/>The [JSON schema](https://json-schema.org/) defines a standardized declaration of the output structure,
     * providing well-defined output documentation that is both human- and machine-readable.
     *
     * @returns {Promise<Record<string, any>>} The [JSON result schema](https://json-schema.org/) of the requested analyzer.
     *
     * @example <caption>Code example</caption>
     * import { analyzersClient } from "@dynatrace-sdk/client-davis-analyzers";
     *
     * const data = await analyzersClient.getJsonSchemaForResult({
     *   analyzerName: "...",
     * });
     *
     * @throws {AnalyzerErrorEnvelopeError} Invalid or malformed request. | Insufficient permissions. | No analyzer available with the requested name. | Client error. | Internal server error.
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    getJsonSchemaForResult(config: {
        /** The name of the analyzer. */ analyzerName: string;
        abortSignal?: AbortSignal;
    }): Promise<Record<string, any>>;
    /**
     * Execute an analyzer with the given input.
     *
     * **Required scope:** davis:analyzers:execute<br/><br/>The analyzer execution is started asynchronously.
     * If the result is not available after the specified timeout, a request token is returned
     * to poll for the result. Otherwise, the final result is directly available and no request token is returned.
     *
     * Depending on the chosen analyzer and the analyzer input, additional scopes might be required, e.g.
     * `storage:buckets:read` and `storage:metrics:read` to enable the analyzer to read time series data from Grail™.
     *
     * @returns {Promise<AnalyzerExecuteResult>} The final analyzer result and execution status. | The current analyzer execution status and a request token and time-to-live (TTL) to continue polling the result with.
     *
     * @example <caption>Code example</caption>
     * import { analyzersClient } from "@dynatrace-sdk/client-davis-analyzers";
     *
     * const data = await analyzersClient.executeAnalyzer({
     *   analyzerName: "...",
     *   body: {
     *     timeSeriesData: {
     *       expression: "timeseries avg(dt.host.cpu.usage)",
     *     },
     *     forecastHorizon: 10,
     *   },
     * });
     *
     * @throws {AnalyzerErrorEnvelopeError} Invalid or malformed request. | Insufficient permissions. | No analyzer available with the requested name. | Client error. | Internal server error.
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    executeAnalyzer(config: {
        body: AnalyzerInput;
        /** The name of the analyzer. */ analyzerName: string;
        /**
         * The amount of seconds to wait for the response to be returned.
         *
         * The timeout controls how long the client is willing to wait, while the analyzer execution continues
         * running until it has finished or is cancelled.
         */ timeoutSeconds?: number;
        /**
         * Indicates if analyzer results should preview already created outputs
         * while the analyzer is still running.
         *
         * Set to false if only the final result is needed.
         */ enablePreview?: boolean;
        abortSignal?: AbortSignal;
    }): Promise<AnalyzerExecuteResult>;
    /**
     * Poll for the result of a started analyzer execution.
     *
     * **Required scope:** davis:analyzers:execute<br/><br/>Use the request token returned when starting an analyzer execution to poll for a result.
     *
     * The `executionStatus` of the result indicates if the analyzer execution is already completed.
     * While the status is `RUNNING`, wait and poll for the result again in subsequent calls.
     *
     * @returns {Promise<AnalyzerPollResult>} The current analyzer execution status and an optional partial result.
     *
     * @example <caption>Code example</caption>
     * import { analyzersClient } from "@dynatrace-sdk/client-davis-analyzers";
     *
     * const data = await analyzersClient.pollAnalyzerExecution({
     *   analyzerName: "...",
     *   requestToken: "RHluYXRyYWNlMjAyMw==",
     * });
     *
     * @throws {AnalyzerErrorEnvelopeError} Invalid or malformed request. | Insufficient permissions. | No analyzer available with the requested name. | The analyzer result with the request token has already been consumed or its time-to-live (TTL) has been reached. The result is not available anymore. | Client error. | Internal server error.
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    pollAnalyzerExecution(config: {
        /** The name of the analyzer. */ analyzerName: string;
        /**
         * The request token is returned when starting a long-running analyzer execution.
         * It can be used to do follow-up operations like polling and cancelling.
         */ requestToken: string;
        /**
         * The amount of seconds to wait for the response to be returned.
         *
         * The timeout controls how long the client is willing to wait, while the analyzer execution continues
         * running until it has finished or is cancelled.
         */ timeoutSeconds?: number;
        abortSignal?: AbortSignal;
    }): Promise<AnalyzerPollResult>;
    /**
     * Stop a started analyzer execution.
     *
     * **Required scope:** davis:analyzers:execute<br/><br/>Use the request token that was returned when starting an analyzer execution to cancel it.
     *
     * @returns {Promise<AnalyzerCancelResult | void>} The current analyzer execution status and an optional partial result.
     *
     * @example <caption>Code example</caption>
     * import { analyzersClient } from "@dynatrace-sdk/client-davis-analyzers";
     *
     * const data = await analyzersClient.cancelAnalyzerExecution({
     *   analyzerName: "...",
     *   requestToken: "RHluYXRyYWNlMjAyMw==",
     * });
     *
     * @throws {AnalyzerErrorEnvelopeError} Invalid or malformed request. | Insufficient permissions. | No analyzer available with the requested name. | The analyzer result with the request token has already been consumed or its time-to-live (TTL) has been reached. The result is not available anymore. | Client error. | Internal server error.
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    cancelAnalyzerExecution(config: {
        /** The name of the analyzer. */ analyzerName: string;
        /**
         * The request token is returned when starting a long-running analyzer execution.
         * It can be used to do follow-up operations like polling and cancelling.
         */ requestToken: string;
        abortSignal?: AbortSignal;
    }): Promise<AnalyzerCancelResult | void>;
    /**
     * Validate the provided input for an analyzer execution.
     *
     * **Required scope:** davis:analyzers:execute<br/><br/>Provide the same input that is planned to be used for an analyzer execution to have it validated without
     * executing the analyzer.
     *
     * @returns {Promise<AnalyzerValidationResult>} The validation result for the requested analyzer execution.
     *
     * @example <caption>Code example</caption>
     * import { analyzersClient } from "@dynatrace-sdk/client-davis-analyzers";
     *
     * const data =
     *   await analyzersClient.validateAnalyzerExecution({
     *     analyzerName: "...",
     *     body: {},
     *   });
     *
     * @throws {AnalyzerErrorEnvelopeError} No analyzer available with the requested name. | Insufficient permissions. | Not found | Client error. | Internal server error.
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    validateAnalyzerExecution(config: {
        body: AnalyzerInput;
        /** The name of the analyzer. */ analyzerName: string;
        abortSignal?: AbortSignal;
    }): Promise<AnalyzerValidationResult>;
}
export declare const analyzersClient: AnalyzersClient;
