import { AbortSignal, HttpClient } from '@dynatrace-sdk/http-client';
import { AddDirectShareRecipients } from '../models/add-direct-share-recipients';
import { CreateDirectShare } from '../models/create-direct-share';
import { DirectShare } from '../models/direct-share';
import { DirectShareList } from '../models/direct-share-list';
import { DirectShareRecipientList } from '../models/direct-share-recipient-list';
import { RemoveDirectShareRecipients } from '../models/remove-direct-share-recipients';
import { ApiClientOptions } from './api-client-options';
/**
 * Share your documents directly with specific users and groups.
 */
export declare class DirectSharesClient {
    private httpClient;
    private shouldTransformDates;
    /**
     * @param {HttpClient} httpClientImplementation - You can provide custom http client as a parameter to constructor of a DirectSharesClient. Custom http client should implement {HttpClient} interface.
     * @param {ApiClientOptions} [options] - Optional configuration for SDK Client.
     * @example
     * const directSharesClientNodeJs = new DirectSharesClient(yourCustomImplementation);
     */
    constructor(httpClientImplementation: HttpClient, options?: ApiClientOptions);
    /**
     * List your direct-shares.
     *
     * **Required scope:** document:direct-shares:read<br/><br/>List all shares of your documents.
     *
     * If you are only concerned with a specific document's shares, or a specific share, you can use the `filter` parameter to narrow down the result set. This does *not* circumvent the restriction of being able to only get shares of *your own* documents. If you attempt to retrieve shares of another user's document, the result set will be empty.
     *
     * *Note, that at the moment we offer a naive pagination, and therefore interim mutations can lead to result inconsistencies (such as duplicates, missing entries).*
     *
     * @returns {Promise<DirectShareList>} A list of your direct-shares.
     *
     * @example <caption>Code example</caption>
     * import { directSharesClient } from "@dynatrace-sdk/client-document";
     *
     * const data = await directSharesClient.listDirectShares();
     *
     * @throws {BadRequest} Malformed request or invalid parameters.
     * @throws {Unauthorized} API token or tenant missing or corrupt.
     * @throws {Forbidden} Access forbidden. This usually happens because the user lacks the permission to access the specific endpoint, or because the target entity is not accessible to the user.
     * @throws {InternalServerError} There is a problem in the backend.
     * @throws {ServiceUnavailable} There is a temporary problem in the backend.
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    listDirectShares(config?: {
        /** The filter query, as explained [here](https://dt-url.net/platform-services-filtering). Filtering is only possible on the `documentId` property, and only with the equals operator. Via this you can effectively request all shares of a specific document owned by you. If this parameter is omitted, all direct-shares of this user will be returned. */ filter?: string;
        /**
         * The page key is used to query results from the next page.
         * You get a `nextPageKey` parameter in the return value of this method to use here.
         * If this parameter is omitted, the first page will be returned.
         */ pageKey?: string;
        /**
         * The page parameter is used to directly access a specific page.
         * The value of the page parameter, if specified, has to be a value greater than zero.
         * If the value of the page parameter exceeds the highest available page on the backend, an empty page is returned.
         */ page?: number;
        /**
         * The page size which defines the requested number of result entries. You can request a maximum of 1000 result entries.
         * If this parameter is omitted, the default value of 20 will be used.
         */ pageSize?: number;
        abortSignal?: AbortSignal;
    }): Promise<DirectShareList>;
    /**
     * Create a direct-share for one of your own documents.
     *
     * **Required scope:** document:direct-shares:write<br/><br/>Create a direct-share for a document owned by you. The share can be used to grant access to a specific set of users and/or groups, via [addRecipients](#/Direct%20Shares/addUserShareRecipients)
     *
     * You can optionally add users and/or groups which will directly be registered as recipients of the share. The users and groups are specified via their sso-ids. The maximum number of recipients is 1000.
     *
     * The validity of the SSO-users and SSO-groups is not verified. It's technically possible, albeit pointless, to add non-existing users and groups.
     *
     * The share can be created with either `read` or `read-write` access.
     *
     * A document can have maximally one direct-share per access type, therefore it's not possible to create multiple `read`-shares or multiple `read-write`-shares for a single document.
     *
     * This means, that you can create one `read`-share for a document, and this single `read`-share can be used to give read-access to an arbitrary number of users (and/or groups). The same applies to a `read-write`-share.
     *
     * @returns {Promise<DirectShare>} A new share for the specified document has been created.
     *
     * @example <caption>Code example</caption>
     * import { directSharesClient } from "@dynatrace-sdk/client-document";
     *
     * const data = await directSharesClient.createDirectShare({
     *   body: {
     *     documentId: "...",
     *     access: "...",
     *     recipients: [
     *       {
     *         id: "441664f0-23c9-40ef-b344-18c02c23d789",
     *         type: "group",
     *       },
     *     ],
     *   },
     * });
     *
     * @throws {BadRequest} Malformed request or invalid parameters.
     * @throws {Unauthorized} API token or tenant missing or corrupt.
     * @throws {Forbidden} Access forbidden. This usually happens because the user lacks the permission to access the specific endpoint, or because the target entity is not accessible to the user.
     * @throws {DocumentNotFound} Document not found.
     * @throws {ShareAlreadyExists} Share creation failed - a share with the specified permission already exists for the document.
     * @throws {InternalServerError} There is a problem in the backend.
     * @throws {ServiceUnavailable} There is a temporary problem in the backend.
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    createDirectShare(config: {
        body: CreateDirectShare;
        abortSignal?: AbortSignal;
    }): Promise<DirectShare>;
    /**
     * Retrieve one of your direct-shares.
     *
     * **Required scope:** document:direct-shares:read<br/><br/>Retrieve a direct-share via its id.
     *
     * Only the share's owner is permitted to do this.
     *
     * @returns {Promise<DirectShare>} A direct-share.
     *
     * @example <caption>Code example</caption>
     * import { directSharesClient } from "@dynatrace-sdk/client-document";
     *
     * const data = await directSharesClient.getDirectShare({
     *   id: "...",
     * });
     *
     * @throws {Unauthorized} API token or tenant missing or corrupt.
     * @throws {Forbidden} Access forbidden. This usually happens because the user lacks the permission to access the specific endpoint, or because the target entity is not accessible to the user.
     * @throws {ShareNotFound} Share not found.
     * @throws {InternalServerError} There is a problem in the backend.
     * @throws {ServiceUnavailable} There is a temporary problem in the backend.
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    getDirectShare(config: {
        /** System-generated id of a share. */ id: string;
        abortSignal?: AbortSignal;
    }): Promise<DirectShare>;
    /**
     * Delete one of your direct-shares.
     *
     * **Required scope:** document:direct-shares:delete<br/><br/>Delete the share. This will *not* delete the share's document.
     *
     * You can only delete shares of your own documents.
     *
     * This operation effectively revokes the access of all of the share's recipients.
     *
     * Be aware that deleting a share does not necessarily prevent a user from accessing a document, as the user might still have access via another share (of the same document). E.g., if a user has 'read' and 'read-write' access (via one 'read' and another 'read-write' share), and the 'read' share gets deleted, access is still granted to the user via the other 'read-write' share.
     *
     * @example <caption>Code example</caption>
     * import { directSharesClient } from "@dynatrace-sdk/client-document";
     *
     * const data = await directSharesClient.deleteDirectShare({
     *   id: "...",
     * });
     *
     * @throws {Unauthorized} API token or tenant missing or corrupt.
     * @throws {Forbidden} Access forbidden. This usually happens because the user lacks the permission to access the specific endpoint, or because the target entity is not accessible to the user.
     * @throws {ShareNotFound} Share not found.
     * @throws {InternalServerError} There is a problem in the backend.
     * @throws {ServiceUnavailable} There is a temporary problem in the backend.
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    deleteDirectShare(config: {
        /** System-generated id of a share. */ id: string;
        abortSignal?: AbortSignal;
    }): Promise<void>;
    /**
     * List the recipients of one of your direct-shares.
     *
     * **Required scope:** document:direct-shares:read<br/><br/>Retrieve a share's recipients. If there are groups among the recipieints, the groups always appear before the users.
     *
     * Only share's owner is permitted to do this.
     *
     * @returns {Promise<DirectShareRecipientList>} A list of recipients of the direct-share.
     *
     * @example <caption>Code example</caption>
     * import { directSharesClient } from "@dynatrace-sdk/client-document";
     *
     * const data =
     *   await directSharesClient.getDirectShareRecipients({
     *     id: "...",
     *   });
     *
     * @throws {Unauthorized} API token or tenant missing or corrupt.
     * @throws {Forbidden} Access forbidden. This usually happens because the user lacks the permission to access the specific endpoint, or because the target entity is not accessible to the user.
     * @throws {ShareNotFound} Share not found.
     * @throws {InternalServerError} There is a problem in the backend.
     * @throws {ServiceUnavailable} There is a temporary problem in the backend.
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    getDirectShareRecipients(config: {
        /** System-generated id of a share. */ id: string;
        abortSignal?: AbortSignal;
    }): Promise<DirectShareRecipientList>;
    /**
     * Add recipients to the share.
     *
     * **Required scope:** document:direct-shares:write<br/><br/>Add one or multiple SSO-users and/or SSO-groups to this share. The affected users immediately gain access to the document.
     *
     * Only share's owner is permitted to do this. The maximum number of recipients is 1000.
     *
     * The validity of the SSO-users and SSO-groups is not verified. It's technically possible, albeit pointless, to add non-existing users and groups.
     *
     * Already added users or groups are ignored.
     *
     * @example <caption>Code example</caption>
     * import { directSharesClient } from "@dynatrace-sdk/client-document";
     *
     * const data =
     *   await directSharesClient.addDirectShareRecipients({
     *     id: "...",
     *     body: {
     *       recipients: [
     *         {
     *           id: "441664f0-23c9-40ef-b344-18c02c23d789",
     *           type: "group",
     *         },
     *       ],
     *     },
     *   });
     *
     * @throws {BadRequest} Malformed request or invalid parameters.
     * @throws {Unauthorized} API token or tenant missing or corrupt.
     * @throws {Forbidden} Access forbidden. This usually happens because the user lacks the permission to access the specific endpoint, or because the target entity is not accessible to the user.
     * @throws {ShareNotFound} Share not found.
     * @throws {InternalServerError} There is a problem in the backend.
     * @throws {ServiceUnavailable} There is a temporary problem in the backend.
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    addDirectShareRecipients(config: {
        body: AddDirectShareRecipients;
        /** System-generated id of a share. */ id: string;
        abortSignal?: AbortSignal;
    }): Promise<void>;
    /**
     * Remove recipients from the share.
     *
     * **Required scope:** document:direct-shares:write<br/><br/>Remove one or multiple recipients from the share. The affected users immediately lose access to the document.
     *
     * Only share's owner is permitted to do this. The maximum number of recipients is 1000.
     *
     * Non-existing users or groups are ignored.
     *
     * @example <caption>Code example</caption>
     * import { directSharesClient } from "@dynatrace-sdk/client-document";
     *
     * const data =
     *   await directSharesClient.removeDirectShareRecipients({
     *     id: "...",
     *     body: { ids: ["..."] },
     *   });
     *
     * @throws {BadRequest} Malformed request or invalid parameters.
     * @throws {Unauthorized} API token or tenant missing or corrupt.
     * @throws {Forbidden} Access forbidden. This usually happens because the user lacks the permission to access the specific endpoint, or because the target entity is not accessible to the user.
     * @throws {ShareNotFound} Share not found.
     * @throws {InternalServerError} There is a problem in the backend.
     * @throws {ServiceUnavailable} There is a temporary problem in the backend.
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    removeDirectShareRecipients(config: {
        body: RemoveDirectShareRecipients;
        /** System-generated id of a share. */ id: string;
        abortSignal?: AbortSignal;
    }): Promise<void>;
}
export declare const directSharesClient: DirectSharesClient;
