import { AbortSignal, HttpClient } from '@dynatrace-sdk/http-client';
import { AppState } from '../models/app-state';
import { AppStates } from '../models/app-states';
import { UserAppState } from '../models/user-app-state';
import { UserAppStates } from '../models/user-app-states';
import { ApiClientOptions } from './api-client-options';
export declare class StateClient {
    private httpClient;
    private shouldTransformDates;
    /**
     * @param {HttpClient} httpClientImplementation - You can provide custom http client as a parameter to constructor of a StateClient. Custom http client should implement {HttpClient} interface.
     * @param {ApiClientOptions} [options] - Optional configuration for SDK Client.
     * @example
     * const stateClientNodeJs = new StateClient(yourCustomImplementation);
     */
    constructor(httpClientImplementation: HttpClient, options?: ApiClientOptions);
    /**
     * List app states
     *
     * **Required scope:** state:app-states:read<br/><br/>Lists app states. By default, only provides property __key__ per state. Use *add-fields* parameter to include more fields and the *filter* parameter to narrow down the returned states.
     *
     * @returns {Promise<AppStates>} The list of app states
     *
     * @example <caption>Code example</caption>
     * import { stateClient } from "@dynatrace-sdk/client-state";
     *
     * const data = await stateClient.getAppStates();
     *
     * @throws {Unauthorized} Unauthorized
     * @throws {Forbidden} Forbidden
     * @throws {InternalServerError} Internal server error
     * @throws {GeneralError} Unexpected error
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    getAppStates(config?: {
        /** Provide a comma separated list of additional properties to be included in the response. */ addFields?: string;
        /**
         * The filter parameter for filtering the set of returned resources
         * If this parameter is omitted, no filtering is applied and all states will be returned.
         *
         *  Filtering by string type parameters `key`, `modificationInfo.lastModifiedBy` when using one of the operators `contains`, `starts-with` and `ends-with` is **case insensitive**.
         *
         * When using the operators `=`and `!=`, filtering is **case sensitive**.
         *
         * The following fields are legal filtering parameters - any other field names will result in a HTTP 400 response:
         *
         * * `key`, supported operators: `=`, `!=`, `contains`, `starts-with`, `ends-with`:
         *
         * * `modificationInfo.lastModifiedTime`, supported operators:	`=`, `!=`, `<`, `<=`, `>`, `>=`
         *
         * * `modificationInfo.lastModifiedBy`, supported operators: `=`, `!=`, `contains`, `starts-with`, `ends-with`
         *
         *
         * * `validUntilTime`, supported operators:	`=`, `!=`, `<`, `<=`, `>`, `>=`
         *
         * The following constraints apply:
         * * Field names are **case-sensitive**.
         * * Conditions can be connected via operators `and` and `or`. A single condition can be negated by `not`.
         * * Strings **must** be enclosed in **single quotes**. e.g. `key contains 'my-string'`
         * * Single quotes within a string must be escaped with a backslash: e.g. `key starts-with 'it\'s a string'`
         * * Maximum nesting depth (via brackets) is 2.
         *
         * * Maximum length is 256 characters.
         *
         * Examples:
         *
         * * `key starts-with 'game-'`
         *
         * * `modificationInfo.lastModifiedTime >= '2022-07-01T00:10:05.000Z' and not (key contains 'new')`
         */ filter?: string;
        abortSignal?: AbortSignal;
    }): Promise<AppStates>;
    /**
     * Delete all app states
     *
     * **Required scope:** state:app-states:delete<br/><br/>Deletes all app states for an app to reset the app into a clean state.
     *
     * @example <caption>Code example</caption>
     * import { stateClient } from "@dynatrace-sdk/client-state";
     *
     * const data = await stateClient.deleteAppStates();
     *
     * @throws {GeneralError} Unexpected error
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    deleteAppStates(config?: {
        abortSignal?: AbortSignal;
    } | AbortSignal): Promise<void>;
    /**
     * Gets app state
     *
     * **Required scope:** state:app-states:read
     *
     * @returns {Promise<AppState>} The app state
     *
     * @example <caption>Code example</caption>
     * import { stateClient } from "@dynatrace-sdk/client-state";
     *
     * const data = await stateClient.getAppState({
     *   key: "some-key",
     * });
     *
     * @throws {Unauthorized} Unauthorized
     * @throws {Forbidden} Forbidden
     * @throws {NotFound} Not found
     * @throws {InternalServerError} Internal server error
     * @throws {GeneralError} Unexpected error
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    getAppState(config: {
        /** Specify the key of the state */ key: string;
        abortSignal?: AbortSignal;
    }): Promise<AppState>;
    /**
     * Updates app state
     *
     * **Required scope:** state:app-states:write<br/><br/>Updates the *cross-user* app state for the given key. Be aware that other users will be able to read the value.
     * Use the user-scoped [user-app-state](#/Public/setUserAppState) to only store values for the authenticated user.
     *
     * **[Certain limits](https://dt-url.net/platform-services-state-service/#limitations)** apply when updating app states.
     *
     * @example <caption>Code example</caption>
     * import { stateClient } from "@dynatrace-sdk/client-state";
     *
     * const data = await stateClient.setAppState({
     *   key: "some-key",
     *   body: { value: "some-state", validUntilTime: "now+2d" },
     * });
     *
     * @throws {AppStateLimitsExceeded} Exceeded size limit for combined size of app states of this app
     * @throws {Unauthorized} Unauthorized
     * @throws {Forbidden} Forbidden
     * @throws {InternalServerError} Internal server error
     * @throws {GeneralError} Unexpected error
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    setAppState(config: {
        body: AppState;
        /** Specify the key of the state */ key: string;
        abortSignal?: AbortSignal;
    }): Promise<void>;
    /**
     * Deletes app state
     *
     * **Required scope:** state:app-states:delete
     *
     * @example <caption>Code example</caption>
     * import { stateClient } from "@dynatrace-sdk/client-state";
     *
     * const data = await stateClient.deleteAppState({
     *   key: "some-key",
     * });
     *
     * @throws {Unauthorized} Unauthorized
     * @throws {Forbidden} Forbidden
     * @throws {NotFound} Not found
     * @throws {InternalServerError} Internal server error
     * @throws {GeneralError} Unexpected error
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    deleteAppState(config: {
        /** Specify the key of the state */ key: string;
        abortSignal?: AbortSignal;
    }): Promise<void>;
    /**
     * List user app states
     *
     * **Required scope:** state:user-app-states:read<br/><br/>Lists user app states. By default, only provides property __key__ per state. Use *add-fields* parameter to include more fields and the *filter* parameter to narrow down the returned states
     *
     * @returns {Promise<UserAppStates>} The list of user app states
     *
     * @example <caption>Code example</caption>
     * import { stateClient } from "@dynatrace-sdk/client-state";
     *
     * const data = await stateClient.getUserAppStates();
     *
     * @throws {Unauthorized} Unauthorized
     * @throws {Forbidden} Forbidden
     * @throws {InternalServerError} Internal server error
     * @throws {GeneralError} Unexpected error
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    getUserAppStates(config?: {
        /** Provide a comma separated list of additional properties to be included in the response. */ addFields?: string;
        /**
         * The filter parameter for filtering the set of returned resources
         * If this parameter is omitted, no filtering is applied and all states will be returned.
         *
         *  Filtering by string type parameters `key`, `modificationInfo.lastModifiedBy` when using one of the operators `contains`, `starts-with` and `ends-with` is **case insensitive**.
         *
         * When using the operators `=`and `!=`, filtering is **case sensitive**.
         *
         * The following fields are legal filtering parameters - any other field names will result in a HTTP 400 response:
         *
         * * `key`, supported operators: `=`, `!=`, `contains`, `starts-with`, `ends-with`:
         *
         * * `modificationInfo.lastModifiedTime`, supported operators:	`=`, `!=`, `<`, `<=`, `>`, `>=`
         *
         * * `modificationInfo.lastModifiedBy`, supported operators: `=`, `!=`, `contains`, `starts-with`, `ends-with`
         *
         *
         * * `validUntilTime`, supported operators:	`=`, `!=`, `<`, `<=`, `>`, `>=`
         *
         * The following constraints apply:
         * * Field names are **case-sensitive**.
         * * Conditions can be connected via operators `and` and `or`. A single condition can be negated by `not`.
         * * Strings **must** be enclosed in **single quotes**. e.g. `key contains 'my-string'`
         * * Single quotes within a string must be escaped with a backslash: e.g. `key starts-with 'it\'s a string'`
         * * Maximum nesting depth (via brackets) is 2.
         *
         * * Maximum length is 256 characters.
         *
         * Examples:
         *
         * * `key starts-with 'game-'`
         *
         * * `modificationInfo.lastModifiedTime >= '2022-07-01T00:10:05.000Z' and not (key contains 'new')`
         */ filter?: string;
        abortSignal?: AbortSignal;
    }): Promise<UserAppStates>;
    /**
     * Delete all user app states
     *
     * **Required scope:** state:user-app-states:delete<br/><br/>Deletes all user app states for the calling user and app.
     *
     * @example <caption>Code example</caption>
     * import { stateClient } from "@dynatrace-sdk/client-state";
     *
     * const data = await stateClient.deleteUserAppStates();
     *
     * @throws {GeneralError} Unexpected error
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    deleteUserAppStates(config?: {
        abortSignal?: AbortSignal;
    } | AbortSignal): Promise<void>;
    /**
     * Get user app state
     *
     * **Required scope:** state:user-app-states:read
     *
     * @returns {Promise<UserAppState>} The user app state
     *
     * @example <caption>Code example</caption>
     * import { stateClient } from "@dynatrace-sdk/client-state";
     *
     * const data = await stateClient.getUserAppState({
     *   key: "some-key",
     * });
     *
     * @throws {Unauthorized} Unauthorized
     * @throws {Forbidden} Forbidden
     * @throws {NotFound} Not found
     * @throws {InternalServerError} Internal server error
     * @throws {GeneralError} Unexpected error
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    getUserAppState(config: {
        /** Specify the key of the state */ key: string;
        abortSignal?: AbortSignal;
    }): Promise<UserAppState>;
    /**
     * Updates user app state
     *
     * **Required scope:** state:user-app-states:write<br/><br/>Updates the user specific app state for the given key and calling user.
     *
     * **[Certain limits](https://dt-url.net/platform-services-state-service/#limitations)** apply when updating user app states.
     *
     * @example <caption>Code example</caption>
     * import { stateClient } from "@dynatrace-sdk/client-state";
     *
     * const data = await stateClient.setUserAppState({
     *   key: "some-key",
     *   body: { value: "some-state", validUntilTime: "now+2d" },
     * });
     *
     * @throws {UserAppStateLimitsExceeded} Exceeded limit for number of user app states to be stored for this user and app
     * @throws {Unauthorized} Unauthorized
     * @throws {Forbidden} Forbidden
     * @throws {InternalServerError} Internal server error
     * @throws {GeneralError} Unexpected error
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    setUserAppState(config: {
        body: UserAppState;
        /** Specify the key of the state */ key: string;
        abortSignal?: AbortSignal;
    }): Promise<void>;
    /**
     * Delete user app state
     *
     * **Required scope:** state:user-app-states:delete
     *
     * @example <caption>Code example</caption>
     * import { stateClient } from "@dynatrace-sdk/client-state";
     *
     * const data = await stateClient.deleteUserAppState({
     *   key: "some-key",
     * });
     *
     * @throws {Unauthorized} Unauthorized
     * @throws {Forbidden} Forbidden
     * @throws {NotFound} Not found
     * @throws {InternalServerError} Internal server error
     * @throws {GeneralError} Unexpected error
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    deleteUserAppState(config: {
        /** Specify the key of the state */ key: string;
        abortSignal?: AbortSignal;
    }): Promise<void>;
}
export declare const stateClient: StateClient;
