import type { ExecuteRequest, FilterSegment, FilterSegments, FilterSegmentVariableDefinition, queryExecutionClient, QueryOptions, QueryPollResponse, QueryResult, ResultRecord } from '@dynatrace-sdk/client-query';
type ExecuteConfig = Omit<Parameters<typeof queryExecutionClient.queryExecute>[0], 'body' | 'abortSignal'>;
/**
 * Parameters for query execution.
 */
export interface DqlQueryParams extends ExecuteConfig, ExecuteRequest {
}
/**
 * Options for the refetch functions returned by the useDql hook.
 */
export interface RefetchOptions {
    /**
     * If set to `true`, a currently running request will be cancelled before a new request is made
     *
     * If set to `false`, no refetch will be made if there is already a request running.
     *
     * Defaults to `true`.
     */
    cancelRefetch?: boolean;
    throwOnError?: boolean;
}
/**
 * Represents a base result from the useDql hook.
 * @template [T=ResultRecord] - The type of data returned by the DQL query.
 * @ignore
 */
export interface UseDqlBaseResult<T = ResultRecord> {
    /**
     * Possible state of the query.
     */
    state: QueryPollResponse['state'];
    /**
     * The progress of the query from 0 to 100.
     */
    progress: QueryPollResponse['progress'];
    /**
     * Function to call to cancel to ongoing query
     */
    cancel: () => Promise<void>;
    /**
     * Function to manually refetch the query.
     */
    refetch: (options?: RefetchOptions) => Promise<UseDqlResult<T>>;
    /**
     * Function to manually force refetch the query. Ignores stale time.
     */
    forceRefetch: (options?: RefetchOptions) => Promise<UseDqlResult<T>>;
    /**
     * `true` if there's no cached data and no query attempt was finished yet.
     */
    isPending: boolean;
    /**
     * `true` if the query has received a response with no errors and is ready to display the data.
     */
    isSuccess: boolean;
    /**
     * `true` if the query attempt resulted in an error.
     */
    isError: boolean;
    /**
     * `true` whenever the first fetch for a query is in-flight.
     */
    isLoading: boolean;
    /**
     * Will be `true` if the query failed while fetching for the first time.
     */
    isLoadingError: boolean;
    /**
     * `true` whenever the query is executing, which includes initial pending as well as background refetches.
     */
    isFetching: boolean;
    /**
     * `true` if the query failed while refetching.
     */
    isRefetchError: boolean;
    /**
     * `true` if the query has been fetched.
     */
    isFetched: boolean;
    /**
     * `true` if the data in the cache is invalidated or if the data is older than the given `staleTime`.
     */
    isStale: boolean;
    /**
     * The last successfully resolved data for the query.
     */
    data?: TypedQueryResult<T>;
    /**
     * The error object for the query, if an error was thrown.
     */
    error: Error | null;
}
/**
 * Represents a successful result from the useDql hook.
 * @template [T=ResultRecord] - The type of data returned by the DQL query.
 * @extends UseDqlBaseResult
 * @ignore
 */
export interface UseDqlSuccessResult<T = ResultRecord> extends UseDqlBaseResult<T> {
    /**
     * The last successfully resolved data for the query.
     */
    data: TypedQueryResult<T>;
    error: null;
    isError: false;
    /**
     * `true` if the query has received a response with no errors and is ready to display the data.
     */
    isSuccess: true;
    isLoading: false;
    isPending: false;
}
/**
 * Represents an error result from the useDql hook.
 * @template [T=ResultRecord] - The type of data returned by the DQL query.
 * @extends UseDqlBaseResult
 * @ignore
 */
export interface UseDqlErrorResult<T = ResultRecord> extends UseDqlBaseResult<T> {
    data: never;
    /**
     * The error object for the query, if an error was thrown.
     */
    error: Error;
    /**
     * `true` if the query attempt resulted in an error.
     */
    isError: true;
    isSuccess: false;
    isLoading: false;
    isPending: false;
}
/**
 * Represents a loading state for the useDql hook.
 * @template [T=ResultRecord] - The type of data returned by the DQL query.
 * @extends UseDqlBaseResult
 * @ignore
 */
export interface UseDqlLoadingResult<T = ResultRecord> extends UseDqlBaseResult<T> {
    data?: TypedQueryResult<T>;
    error: null;
    isError: false;
    isSuccess: false;
    /**
     * `true` whenever the first fetch for a query is in-flight.
     */
    isLoading: true;
    isPending: true;
}
/**
 * Represents a pending state for the useDql hook.
 * @template [T=ResultRecord] - The type of data returned by the DQL query.
 * @extends UseDqlBaseResult
 * @ignore
 */
export interface UseDqlPendingResult<T = ResultRecord> extends UseDqlBaseResult<T> {
    data?: TypedQueryResult<T>;
    error: null;
    isError: false;
    isSuccess: false;
    /**
     * `true` if there's no cached data and no query attempt was finished yet.
     */
    isPending: true;
}
/**
 * Represents the possible results of the useDql hook.
 * @template [T=ResultRecord] - The type of data returned by the DQL query.
 * @interface
 */
export type UseDqlResult<T = ResultRecord> = UseDqlSuccessResult<T> | UseDqlErrorResult<T> | UseDqlLoadingResult<T> | UseDqlPendingResult<T>;
/**
 * Generic type that allows to type data returned from Grail.
 * @interface
 */
export type TypedQueryResult<T = ResultRecord> = Omit<QueryResult, 'records'> & {
    records: T[];
};
/**
 * Options for the useDql hook.
 */
export interface UseDqlOptions {
    /**
     * The time in milliseconds after which data is considered stale.
     */
    staleTime?: number;
    /**
     * The rate at which Grail query is polled
     */
    pollRate?: number;
    /**
     * Query will continuously refetch at this frequency in milliseconds
     */
    refetchInterval?: number;
    /**
     * Set this to `false` to disable this query from automatically running. Default: `true`
     */
    enabled?: boolean;
}
export type { FilterSegment, FilterSegments, FilterSegmentVariableDefinition, QueryOptions };
